@@ -42,11 +42,13 @@ class FastMemoryConnection(Connection):
         if not self.read_buffers:
             logger = Logger("network")
             logger("read(%i) EOF", n)
+            time.sleep(0.1)
             return None
         b = self.read_buffers[0]
         if len(b)<=n:
             return self.read_buffers.pop(0)
         self.read_buffers[0] = b[n:]
+        time.sleep(0.05)
         return b[:n]
 
     def write(self, buf, packet_type=None):
@@ -151,7 +153,8 @@ class ProtocolTest(unittest.TestCase):
                 errs.append("not read any raw packets")
             loop.quit()
         loop = GLib.MainLoop()
-        GLib.timeout_add(500, check_failed)
+        # we use a fake `tcp` connection, so the eof will call close() after 1000+500ms:
+        GLib.timeout_add(2500, check_failed)
         GLib.timeout_add(TIMEOUT*1000, loop.quit)
         proto.start()
         loop.run()
