@@ -1,12 +1,13 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # This file is part of Xpra.
 # Copyright (C) 2017 Antoine Martin <antoine@xpra.org>
 
 import os
 import sys
-import pygtk
-pygtk.require('2.0')
-import gtk
+import gi
+gi.require_version("Gtk", "3.0")  # @UndefinedVariable
+gi.require_version("Gdk", "3.0")  # @UndefinedVariable
+from gi.repository import Gtk, Gdk, GdkPixbuf #pylint: disable=wrong-import-position @UnresolvedImport
 
 count = 0
 def handle_owner_change(clipboard, event):
@@ -18,12 +19,12 @@ def handle_owner_change(clipboard, event):
 
 def main():
     assert os.path.exists(sys.argv[1])
-    image = gtk.gdk.pixbuf_new_from_file(sys.argv[1])
-    clipboard = gtk.clipboard_get()
+    image = GdkPixbuf.Pixbuf.new_from_file(sys.argv[1])
+    clipboard = Gtk.Clipboard.get(Gdk.Atom.intern("CLIPBOARD", False))
     clipboard.connect('owner-change', handle_owner_change)
     clipboard.set_image(image)
     clipboard.store()
-    gtk.main()
+    Gtk.main()
 
 if __name__ == "__main__":
     main()
