@@ -517,10 +517,10 @@ class WindowClient(StubClientMixin):
         self.wheel_deltax += deltax
         self.wheel_deltay += deltay
         button = self.wheel_map.get(6+int(self.wheel_deltax>0), 0)            #RIGHT=7, LEFT=6
-        if button>0:
+        if button>0 and deltax:
             self.wheel_deltax = self.send_wheel_delta(device_id, wid, button, self.wheel_deltax, pointer, props)
         button = self.wheel_map.get(5-int(self.wheel_deltay>0), 0)            #UP=4, DOWN=5
-        if button>0:
+        if button>0 and deltay:
             self.wheel_deltay = self.send_wheel_delta(device_id, wid, button, self.wheel_deltay, pointer, props)
         mouselog("wheel_event%s new deltas=%s,%s",
                  (device_id, wid, deltax, deltay), self.wheel_deltax, self.wheel_deltay)
@@ -1374,7 +1374,7 @@ class WindowClient(StubClientMixin):
             return
         (wid, device, percent, pitch, duration, bell_class, bell_id, bell_name) = packet[1:9]
         window = self._id_to_window.get(wid)
-        self.window_bell(window, device, percent, pitch, duration, bell_class, bell_id, bell_name)
+        self.window_bell(window, device, percent, pitch, duration, bell_class, bell_id, bytestostr(bell_name))
 
 
     ######################################################################
