@@ -518,11 +518,11 @@ def get_monitors_info(xscale:float=1, yscale:float=1) -> Dict[int,Any]:
                 if value is None:
                     continue
                 if isinstance(value, Gdk.Rectangle):
-                    value = (round(xscale*value.x), round(yscale*value.y), round(xscale*value.width), round(yscale*value.height))
+                    value = (round(value.x/xscale), round(value.y/yscale), round(value.width/xscale), round(value.height/yscale))
                 elif attr=="width-mm":
-                    value = round(xscale*value)
+                    value = round(value/xscale)
                 elif attr=="height-mm":
-                    value = round(yscale*value)
+                    value = round(value/yscale)
                 elif attr=="subpixel-layout":
                     value = {
                         Gdk.SubpixelLayout.UNKNOWN          : "unknown",
