@@ -174,7 +174,9 @@ class GUI(Gtk.Window):
 
     def reset_cursors(self):
         for widget in self.widgets:
-            widget.get_window().set_cursor(None)
+            gdkwin = widget.get_window()
+            if gdkwin:
+                gdkwin.set_cursor(None)
 
     def busy_cursor(self, widget):
         from xpra.gtk_common.cursor_names import cursor_types
