@@ -1,40 +1,46 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # This file is part of Xpra.
-# Copyright (C) 2013-2019 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2013-2023 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 # taken from the code I wrote for winswitch
 
+import os.path
 import socket
 import sys
+from typing import Dict, Tuple, Any, Optional, Callable
 
-from xpra.os_util import WIN32, OSX, PYTHON3
+from xpra.os_util import WIN32
+from xpra.version_util import parse_version
+from xpra.net.common import FLUSH_HEADER
+from xpra.common import FULL_INFO
 from xpra.log import Logger
 
 log = Logger("network", "util")
 
 
-netifaces_version = 0
+netifaces_version : Tuple[Any, ...] = ()
 _netifaces = None
-def import_netifaces():
+def import_netifaces() -> object:
     global _netifaces, netifaces_version
     if _netifaces is None:
         try:
-            import netifaces                #@UnresolvedImport
-            log("netifaces loaded sucessfully")
+            import netifaces                #@UnresolvedImport pylint: disable=import-outside-toplevel
+            log("netifaces loaded successfully")
             _netifaces = netifaces
-            netifaces_version = netifaces.version        #@UndefinedVariable
+            netifaces_version = parse_version(netifaces.version)    #@UndefinedVariable
         except ImportError:
             _netifaces = False
             log.warn("Warning: the python netifaces package is missing")
+            log.warn(" some networking functionality will be unavailable")
     return _netifaces
 
 iface_ipmasks = {}
 bind_IPs = None
 
 
-def get_free_tcp_port():
+def get_free_tcp_port() -> int:
     s = socket.socket()
     s.bind(('', 0))
     port = s.getsockname()[1]
@@ -46,14 +52,14 @@ def get_interfaces():
     netifaces = import_netifaces()
     if not netifaces:
         return []
-    return netifaces.interfaces()           #@UndefinedVariable
+    return netifaces.interfaces()           #@UndefinedVariable pylint: disable=no-member
 
-def get_interfaces_addresses():
+def get_interfaces_addresses() -> Dict:
     d = {}
     netifaces = import_netifaces()
     if netifaces:
         for iface in get_interfaces():
-            d[iface] = netifaces.ifaddresses(iface)     #@UndefinedVariable
+            d[iface] = netifaces.ifaddresses(iface)     #@UndefinedVariable pylint: disable=no-member
     return d
 
 def get_interface(address):
@@ -70,16 +76,16 @@ def get_interface(address):
                     return iface
     return None
 
-def get_gateways():
+def get_gateways() -> Dict:
     netifaces = import_netifaces()
     if not netifaces:
         return {}
     #versions older than 0.10.5 can crash when calling gateways()
     #https://bitbucket.org/al45tair/netifaces/issues/15/gateways-function-crash-segmentation-fault
-    if netifaces.version<'0.10.5':            #@UndefinedVariable
+    if netifaces.version<'0.10.5':            #@UndefinedVariable pylint: disable=no-member
         return {}
     try:
-        d = netifaces.gateways()            #@UndefinedVariable
+        d = netifaces.gateways()            #@UndefinedVariable pylint: disable=no-member
         AF_NAMES = {}
         for k in dir(netifaces):
             if k.startswith("AF_"):
@@ -106,11 +112,10 @@ def get_bind_IPs():
     return bind_IPs
 
 def do_get_bind_IPs():
-    global iface_ipmasks
     ips = []
     netifaces = import_netifaces()
     assert netifaces
-    ifaces = netifaces.interfaces()            #@UndefinedVariable
+    ifaces = netifaces.interfaces()            #@UndefinedVariable pylint: disable=no-member
     log("ifaces=%s", ifaces)
     for iface in ifaces:
         if_ipmasks = []
@@ -135,7 +140,7 @@ def do_get_bind_ifacemask(iface):
     ipmasks = []
     netifaces = import_netifaces()
     assert netifaces
-    address_types = netifaces.ifaddresses(iface)    #@UndefinedVariable
+    address_types = netifaces.ifaddresses(iface)    #@UndefinedVariable pylint: disable=no-member
     for addresses in address_types.values():
         for address in addresses:
             if 'netmask' in address and 'addr' in address:
@@ -143,68 +148,103 @@ def do_get_bind_ifacemask(iface):
                 mask = address['netmask']
                 if addr!= '::1' and addr != '0.0.0.0' and addr.find("%")<0:
                     try:
-                        socket.inet_aton(addr)
-                        ipmasks.append((addr,mask))
+                        fam = socket.AF_INET6 if len(addr.split(":"))>2 else socket.AF_INET
+                        b = socket.inet_pton(fam, addr)
+                        log(f"socket.inet_pton(AF_INET%s, {addr}={b}", "6" if fam==socket.AF_INET6 else "")
+                        ipmasks.append((addr, mask))
                     except Exception as e:
-                        log("do_get_bind_ifacemask(%s)", iface, exc_info=True)
-                        log.error("Error converting address '%s' to binary, for interface %s", addr, iface)
+                        log(f"do_get_bind_ifacemask({iface})", exc_info=True)
+                        log.error(f"Error converting address {addr!r} with mask {mask!r} to binary")
+                        log.error(f" for interface {iface}:")
+                        log.estr(e)
     log("do_get_bind_ifacemask(%s)=%s", iface, ipmasks)
     return ipmasks
 
-def get_iface(ip):
+def get_iface(ip) -> Optional[str]:
     log("get_iface(%s)", ip)
     if not ip:
         return None
     if ip.find("%")>=0:
-        return ip.split("%", 1)[1]
-    if ip.find(":")>=0:
-        #ipv6?
-        return None
-    if any(x for x in ip if (".:0123456789").find(x)<0):
+        iface = ip.split("%", 1)[1]
+        if if_nametoindex:
+            try:
+                if_nametoindex(iface)
+            except OSError:
+                return None
+            else:
+                return iface
+    ipv6 = ip.find(":")>=0
+    af = socket.AF_INET6 if ipv6 else socket.AF_INET
+    ipchars = ".:0123456789"
+    if ipv6:
+        ipchars += "[]"
+    if any(x for x in ip if ipchars.find(x)<0):
         #extra characters, assume this is a hostname:
+        sockaddr = ()
         try:
             v = socket.getaddrinfo(ip, None)
             assert len(v)>0
         except Exception as e:
-            log.error("Error: cannot revolve '%s'", ip)
+            log("socket.getaddrinfo(%s, None)", ip, exc_info=True)
+            log.error(f"Error resolving {ip!r}: {e}")
             return None
         for i, x in enumerate(v):
             family, socktype, proto, canonname, sockaddr = x
             log("get_iface(%s) [%i]=%s", ip, i, (family, socktype, proto, canonname, sockaddr))
-            if family==socket.AF_INET:
+            if family==af:
                 break
-        log("get_iface(%s) sockaddr=%s", ip, sockaddr)
-        ip = sockaddr[0]
+        if sockaddr:
+            log("get_iface(%s) sockaddr=%s", ip, sockaddr)
+            ip = sockaddr[0]
 
-    ip_parts = ip.split(".")
-    if len(ip_parts)!=4:
-        return None
+    ipv6 = ip.find(":")>=0
+    if ipv6:
+        ip_parts = ip.split(":")
+    else:
+        ip_parts = ip.split(".")
+        if len(ip_parts)!=4:
+            return None
 
     best_match = None
     get_bind_IPs()
-    for (iface, ipmasks) in iface_ipmasks.items():
-        for (test_ip,mask) in ipmasks:
+    for iface, ipmasks in iface_ipmasks.items():
+        for test_ip, mask in ipmasks:
             if test_ip == ip:
                 #exact match
                 log("get_iface(%s)=%s", iface, ip)
                 return iface
-            test_ip_parts = test_ip.split(".")
-            mask_parts = mask.split(".")
-            if len(test_ip_parts)!=4 or len(mask_parts)!=4:
-                log.error("incorrect ip or mask: %s/%s", test_ip, mask)
+            if len(test_ip.split(":"))>2:
+                #test_ip is ipv6
+                test_ip_parts = test_ip.split("/")[0].split(":")
+                mask_parts = mask.split("/")[0].split(":")
+            else:
+                test_ip_parts = test_ip.split(".")
+                mask_parts = mask.split(".")
+                if len(test_ip_parts)!=4 or len(mask_parts)!=4:
+                    log.error(f"Error: incorrect IP {test_ip} or mask {mask}")
             match = True
             try:
-                for i in (0,1,2,3):
-                    mask_part = int(mask_parts[i])
-                    ip_part = int(ip_parts[i]) & mask_part
-                    test_ip_part = int(test_ip_parts[i]) & mask_part
+                for i, ip_part_str in enumerate(test_ip_parts):
+                    if i>=len(mask_parts):
+                        #end of the mask
+                        break
+                    try:
+                        mask_part = int(mask_parts[i] or 0)
+                    except ValueError:
+                        mask_part = int(mask_parts[i], 16)
+                    try:
+                        ip_part = int(ip_part_str or 0)
+                    except ValueError:
+                        ip_part = int(ip_part_str, 16)
+                    test_ip_part = ip_part & mask_part
                     if ip_part!=test_ip_part:
                         match = False
                         break
                 if match:
                     best_match = iface
             except Exception as e:
-                log.error("error parsing ip (%s) or its mask (%s): %s", test_ip, mask, e)
+                log("ip parsing error", exc_info=True)
+                log.error(f"Error parsing IP {test_ip!r} or its mask {mask!r}: {e}")
     log("get_iface(%s)=%s", ip, best_match)
     return best_match
 
@@ -214,7 +254,7 @@ def get_iface(ip):
 if_nametoindex = None
 if_indextoname = None
 
-if WIN32:
+if WIN32:   # pragma: no cover
     def int_if_nametoindex(iface):
         #IPv6 addresses give us the interface as a string:
         #fe80:....%11, so try to convert "11" into 11
@@ -223,7 +263,7 @@ if WIN32:
         except (TypeError, ValueError):
             return None
     if_nametoindex = int_if_nametoindex
-elif PYTHON3:
+else:
     if_nametoindex = socket.if_nametoindex
     def socket_if_indextoname(index):
         if index<0:
@@ -231,129 +271,103 @@ elif PYTHON3:
         return socket.if_indextoname(index)
     if_indextoname = socket_if_indextoname
 
-else:
-    library = "libc.so.6"
-    if OSX:
-        library = "/usr/lib/libc.dylib"
-    elif sys.platform.startswith("sunos"):
-        library = "libsocket.so.1"
-    elif sys.platform.startswith("freebsd"):
-        library = "/lib/libc.so.7"
-    elif sys.platform.startswith("openbsd"):
-        library = "libc.so"
-    try:
-        from ctypes import CDLL, c_char_p, c_uint, create_string_buffer
-        #cdll.LoadLibrary(library)
-        from ctypes.util import find_library
-        #<CDLL 'libc.so.6', handle 7fcac419b000 at 7fcac1ab0c10>
-        _libc = CDLL(find_library(library))
-        log("successfully loaded socket C library from %s", library)
-    except ImportError as e:
-        log.error("library %s not found: %s", library, e)
-    except OSError as e:
-        log.error("error loading %s: %s", library, e)
-    else:
-        _libc.if_indextoname.restype = c_char_p
-        _libc.if_indextoname.argtypes = [c_uint, c_char_p]
-        _libc.if_nametoindex.restype = c_uint
-        _libc.if_nametoindex.argtypes = [c_char_p]
-        def libc_if_nametoindex(interfaceName):
-            return _libc.if_nametoindex(create_string_buffer(interfaceName.encode()))
-        def libc_if_indextoname(index):
-            s = create_string_buffer(b'\000' * 256)
-            return _libc.if_indextoname(c_uint(index), s)
-        if_nametoindex = libc_if_nametoindex
-        if_indextoname = libc_if_indextoname
 
-
-net_sys_config = None
-def get_net_sys_config():
+net_sys_config : Dict[str,Any] = {}
+def get_net_sys_config() -> Dict[str,Any]:
     global net_sys_config
-    if net_sys_config is None:
-        net_sys_config = {}
-        if sys.platform.startswith("linux"):
-            def stripnl(v):
-                return str(v).rstrip("\r").rstrip("\n")
-            def addproc(procpath, subsystem, name, conv=stripnl):
-                assert name
-                try:
-                    with open(procpath) as f:
-                        data = f.read()
-                        subdict = net_sys_config.setdefault(subsystem, {})
-                        if name.find("/")>0:
-                            sub, name = name.split("/", 1)
-                            subdict = subdict.setdefault(sub, {})
-                        for sub in ("ip", "tcp", "ipfrag", "icmp", "igmp"):
-                            if name.startswith("%s_" % sub):
-                                name = name[len(sub)+1:]
-                                subdict = subdict.setdefault(sub, {})
-                                break
-                        subdict[name] = conv(data)
-                except Exception as e:
-                    log("cannot read '%s': %s", procpath, e)
-            for k in ("netdev_max_backlog", "optmem_max", "rmem_default", "rmem_max", "wmem_default", "wmem_max", "max_skb_frags",
-                    "busy_poll", "busy_read", "somaxconn"):
-                addproc("/proc/sys/net/core/%s" % k,     "core", k, int)
-            for k in ("default_qdisc", ):
-                addproc("/proc/sys/net/core/%s" % k,     "core", k)
-            for k in ("max_dgram_qlen", ):
-                addproc("/proc/sys/net/unix/%s" % k,     "unix", k, int)
-            for k in ("ip_forward", "ip_forward_use_pmtu", "tcp_abort_on_overflow", "fwmark_reflect", "tcp_autocorking", "tcp_dsack",
-                    "tcp_ecn_fallback", "tcp_fack",
-                    #"tcp_l3mdev_accept",
-                    "tcp_low_latency", "tcp_no_metrics_save", "tcp_recovery", "tcp_retrans_collapse", "tcp_timestamps",
-                    "tcp_workaround_signed_windows", "tcp_thin_linear_timeouts", "tcp_thin_dupack", "ip_nonlocal_bind",
-                    "ip_dynaddr", "ip_early_demux", "icmp_echo_ignore_all", "icmp_echo_ignore_broadcasts",
-                    ):
-                addproc("/proc/sys/net/ipv4/%s" % k,     "ipv4", k, bool)
-            for k in ("tcp_allowed_congestion_control", "tcp_available_congestion_control", "tcp_congestion_control", "tcp_early_retrans",
-                    "tcp_moderate_rcvbuf", "tcp_rfc1337", "tcp_sack", "tcp_slow_start_after_idle", "tcp_stdurg",
-                    "tcp_syncookies", "tcp_tw_recycle", "tcp_tw_reuse", "tcp_window_scaling",
-                    "icmp_ignore_bogus_error_responses", "icmp_errors_use_inbound_ifaddr"):
-                addproc("/proc/sys/net/ipv4/%s" % k,     "ipv4", k)
-            def parsenums(v):
-                return tuple(int(x.strip()) for x in v.split("\t") if len(x.strip())>0)
-            for k in ("tcp_mem", "tcp_rmem", "tcp_wmem", "ip_local_port_range", "ip_local_reserved_ports", ):
-                addproc("/proc/sys/net/ipv4/%s" % k,     "ipv4", k, parsenums)
-            for k in ("ip_default_ttl", "ip_no_pmtu_disc", "route/min_pmtu",
-                    "route/mtu_expires", "route/min_adv_mss",
-                    "ipfrag_high_thresh", "ipfrag_low_thresh", "ipfrag_time", "ipfrag_max_dist",
-                    "tcp_adv_win_scale", "tcp_app_win", "tcp_base_mss", "tcp_ecn", "tcp_fin_timeout", "tcp_frto",
-                    "tcp_invalid_ratelimit", "tcp_keepalive_time", "tcp_keepalive_probes", "tcp_keepalive_intvl",
-                    "tcp_max_orphans", "tcp_max_syn_backlog", "tcp_max_tw_buckets",
-                    "tcp_min_rtt_wlen", "tcp_mtu_probing", "tcp_probe_interval", "tcp_probe_threshold", "tcp_orphan_retries",
-                    "tcp_reordering", "tcp_max_reordering", "tcp_retries1", "tcp_retries2", "tcp_synack_retries",
-                    "tcp_fastopen", "tcp_syn_retries", "tcp_min_tso_segs", "tcp_pacing_ss_ratio",
-                    "tcp_pacing_ca_ratio", "tcp_tso_win_divisor", "tcp_notsent_lowat",
-                    "tcp_limit_output_bytes", "tcp_challenge_ack_limit",
-                    "icmp_ratelimit", "icmp_msgs_per_sec", "icmp_msgs_burst", "icmp_ratemask",
-                    "igmp_max_memberships", "igmp_max_msf", "igmp_qrv",
-                    ):
-                addproc("/proc/sys/net/ipv4/%s" % k,     "ipv4", k, int)
+    if net_sys_config or not os.path.exists("/proc"):
+        return net_sys_config
+    def stripnl(v):
+        return str(v).rstrip("\r").rstrip("\n")
+    def addproc(procpath:str, subsystem:str, name:str, conv:Callable=stripnl):
+        assert name
+        try:
+            with open(procpath, mode="r", encoding="latin1") as f:
+                data = f.read()
+                subdict = net_sys_config.setdefault(subsystem, {})
+                if name.find("/")>0:
+                    sub, name = name.split("/", 1)
+                    subdict = subdict.setdefault(sub, {})
+                for sub in ("ip", "tcp", "ipfrag", "icmp", "igmp"):
+                    if name.startswith(f"{sub}_"):
+                        name = name[len(sub)+1:]
+                        subdict = subdict.setdefault(sub, {})
+                        break
+                subdict[name] = conv(data)
+        except Exception as e:
+            log("cannot read '%s': %s", procpath, e)
+    for k in ("netdev_max_backlog", "optmem_max",
+              "rmem_default", "rmem_max", "wmem_default", "wmem_max", "max_skb_frags",
+              "busy_poll", "busy_read", "somaxconn",
+              ):
+        addproc(f"/proc/sys/net/core/{k}",  "core", k, int)
+    for k in ("default_qdisc", ):
+        addproc(f"/proc/sys/net/core/{k}",  "core", k)
+    for k in ("max_dgram_qlen", ):
+        addproc(f"/proc/sys/net/unix/{k}",  "unix", k, int)
+    for k in (
+        "ip_forward", "ip_forward_use_pmtu",
+        "tcp_abort_on_overflow", "fwmark_reflect", "tcp_autocorking", "tcp_dsack",
+        "tcp_ecn_fallback", "tcp_fack",
+        #"tcp_l3mdev_accept",
+        "tcp_low_latency", "tcp_no_metrics_save", "tcp_recovery", "tcp_retrans_collapse", "tcp_timestamps",
+        "tcp_workaround_signed_windows", "tcp_thin_linear_timeouts", "tcp_thin_dupack", "ip_nonlocal_bind",
+        "ip_dynaddr", "ip_early_demux", "icmp_echo_ignore_all", "icmp_echo_ignore_broadcasts",
+        ):
+        addproc(f"/proc/sys/net/ipv4/{k}",  "ipv4", k, bool)
+    for k in (
+        "tcp_allowed_congestion_control", "tcp_available_congestion_control",
+        "tcp_congestion_control", "tcp_early_retrans",
+        "tcp_moderate_rcvbuf", "tcp_rfc1337", "tcp_sack", "tcp_slow_start_after_idle", "tcp_stdurg",
+        "tcp_syncookies", "tcp_tw_recycle", "tcp_tw_reuse", "tcp_window_scaling",
+        "icmp_ignore_bogus_error_responses", "icmp_errors_use_inbound_ifaddr",
+        ):
+        addproc(f"/proc/sys/net/ipv4/{k}",  "ipv4", k)
+    def parsenums(v:str) -> Tuple[int,...]:
+        return tuple(int(x.strip()) for x in v.split("\t") if len(x.strip())>0)
+    for k in ("tcp_mem", "tcp_rmem", "tcp_wmem", "ip_local_port_range", "ip_local_reserved_ports", ):
+        addproc(f"/proc/sys/net/ipv4/{k}",  "ipv4", k, parsenums)
+    for k in (
+        "ip_default_ttl", "ip_no_pmtu_disc", "route/min_pmtu",
+        "route/mtu_expires", "route/min_adv_mss",
+        "ipfrag_high_thresh", "ipfrag_low_thresh", "ipfrag_time", "ipfrag_max_dist",
+        "tcp_adv_win_scale", "tcp_app_win", "tcp_base_mss", "tcp_ecn", "tcp_fin_timeout", "tcp_frto",
+        "tcp_invalid_ratelimit", "tcp_keepalive_time", "tcp_keepalive_probes", "tcp_keepalive_intvl",
+        "tcp_max_orphans", "tcp_max_syn_backlog", "tcp_max_tw_buckets",
+        "tcp_min_rtt_wlen", "tcp_mtu_probing", "tcp_probe_interval",
+        "tcp_probe_threshold", "tcp_orphan_retries",
+        "tcp_reordering", "tcp_max_reordering", "tcp_retries1", "tcp_retries2", "tcp_synack_retries",
+        "tcp_fastopen", "tcp_syn_retries", "tcp_min_tso_segs", "tcp_pacing_ss_ratio",
+        "tcp_pacing_ca_ratio", "tcp_tso_win_divisor", "tcp_notsent_lowat",
+        "tcp_limit_output_bytes", "tcp_challenge_ack_limit",
+        "icmp_ratelimit", "icmp_msgs_per_sec", "icmp_msgs_burst", "icmp_ratemask",
+        "igmp_max_memberships", "igmp_max_msf", "igmp_qrv",
+        ):
+        addproc(f"/proc/sys/net/ipv4/{k}",  "ipv4", k, int)
     return net_sys_config
 
-def get_net_config():
+def get_net_config() -> Dict[str,Any]:
     config = {}
     try:
-        from xpra.net.bytestreams import VSOCK_TIMEOUT, SOCKET_TIMEOUT, SOCKET_NODELAY
+        from xpra.net.bytestreams import VSOCK_TIMEOUT, SOCKET_TIMEOUT, SOCKET_NODELAY  # pylint: disable=import-outside-toplevel
         config = {
                 "vsocket.timeout"    : VSOCK_TIMEOUT,
                 "socket.timeout"     : SOCKET_TIMEOUT,
                 }
         if SOCKET_NODELAY is not None:
             config["socket.nodelay"] = SOCKET_NODELAY
-    except Exception:
+    except Exception:   # pragma: no cover
         log("get_net_config()", exc_info=True)
     return config
 
 
-def get_ssl_info(show_constants=False):
+def get_ssl_info(show_constants=False) -> Dict[str,Any]:
     try:
-        import ssl
-    except ImportError as e:
+        import ssl  # pylint: disable=import-outside-toplevel
+    except ImportError as e:    # pragma: no cover
         log("no ssl: %s", e)
         return {}
-    info = {}
+    info : Dict[str,Any] = {}
     if show_constants:
         protocols = dict((k,int(getattr(ssl, k))) for k in dir(ssl) if k.startswith("PROTOCOL_"))
         ops = dict((k,int(getattr(ssl, k))) for k in dir(ssl) if k.startswith("OP_"))
@@ -373,39 +387,38 @@ def get_ssl_info(show_constants=False):
         v = getattr(ssl, k, None)
         if v is not None:
             info[name] = v
-    for k, idef in {
-                    ""           : ("version", str),
-                    "_INFO"      : ("version-info", str),
-                    "_NUMBER"    : ("version-number", int),
-                    }.items():
-        v = getattr(ssl, "OPENSSL_VERSION%s" % k, None)
-        if v is not None:
-            name, conv = idef
-            info.setdefault("openssl", {})[name] = conv(v)
+
+    vnum = getattr(ssl, "OPENSSL_VERSION_NUMBER", 0)
+    if vnum:
+        vparts = []
+        for i in range(4):
+            vparts.append((vnum & 0xff) >> 4)
+            vnum = vnum >> 8
+        info["openssl-version"] = tuple(reversed(vparts))[:FULL_INFO+1]
     return info
 
 
-def get_network_caps():
+def get_network_caps(full_info : int=1) -> Dict[str,Any]:
+    # pylint: disable=import-outside-toplevel
     from xpra.net.digest import get_digests
-    from xpra.net.crypto import get_crypto_caps
     from xpra.net.compression import get_enabled_compressors, get_compression_caps
     from xpra.net.packet_encoding import get_enabled_encoders, get_packet_encoding_caps
     digests = get_digests()
     #"hmac" is the legacy name, "xor" and "des" should not be used for salt:
     salt_digests = tuple(x for x in digests if x not in ("hmac", "xor", "des"))
-    caps = {
-                "digest"                : digests,
-                "salt-digest"           : salt_digests,
-                "compressors"           : get_enabled_compressors(),
-                "encoders"              : get_enabled_encoders(),
-               }
-    caps.update(get_crypto_caps())
-    caps.update(get_compression_caps())
-    caps.update(get_packet_encoding_caps())
+    caps : Dict[str,Any] = {
+        "digest"                : digests,
+        "salt-digest"           : salt_digests,
+        "compressors"           : get_enabled_compressors(),
+        "encoders"              : get_enabled_encoders(),
+        "flush"                 : FLUSH_HEADER,
+    }
+    caps.update(get_compression_caps(full_info))
+    caps.update(get_packet_encoding_caps(full_info))
     return caps
 
 
-def get_info():
+def get_info() -> Dict[str,Any]:
     i = get_network_caps()
     netifaces = import_netifaces()
     if netifaces:
@@ -415,9 +428,10 @@ def get_info():
         ssli = get_ssl_info()
         ssli[""] = True
         i["ssl"] = ssli
-    s = get_net_sys_config()
-    if s:
-        i["system"] = s
+    if FULL_INFO>1:
+        s = get_net_sys_config()
+        if s:
+            i["system"] = s
     i["config"] = get_net_config()
     paramiko = sys.modules.get("paramiko")
     if paramiko:
@@ -427,7 +441,8 @@ def get_info():
     return i
 
 
-def main():
+def main(): # pragma: no cover
+    # pylint: disable=import-outside-toplevel
     from xpra.os_util import POSIX
     from xpra.util import print_nested_dict, csv
     from xpra.platform import program_context
@@ -447,8 +462,8 @@ def main():
             if if_nametoindex:
                 print("* %s (index=%s)" % (iface.ljust(20), if_nametoindex(iface)))
             else:
-                print("* %s" % iface)
-            addresses = netifaces.ifaddresses(iface)     #@UndefinedVariable
+                print(f"* {iface}")
+            addresses = netifaces.ifaddresses(iface)     #@UndefinedVariable pylint: disable=no-member
             for addr, defs in addresses.items():
                 if addr in (socket.AF_INET, socket.AF_INET6):
                     for d in defs:
@@ -472,7 +487,7 @@ def main():
             if not POSIX:
                 info = get_interface_info(0, iface)
                 if info:
-                    print("  %s" % info)
+                    print(f"  {info}")
 
         from xpra.os_util import bytestostr
         def pver(v):
@@ -481,7 +496,7 @@ def main():
                 lastx = None
                 for x in v:
                     if lastx is not None:
-                        #dot seperated numbers
+                        #dot separated numbers
                         if isinstance(lastx, int):
                             s += "."
                         else:
@@ -497,14 +512,17 @@ def main():
 
         print("Gateways found:")
         for gt,idefs in get_gateways().items():
-            print("* %s" % gt)      #ie: "INET"
+            print(f"* {gt}")      #ie: "INET"
             for i, idef in enumerate(idefs):
                 if isinstance(idef, (list, tuple)):
-                    print(" [%i]           %s" % (i, csv(idef)))
-                    continue
+                    print(f" [{i}]           "+csv(idef))
 
         print("")
         print("Protocol Capabilities:")
+        from xpra.net import compression
+        compression.init_all()
+        from xpra.net import packet_encoding
+        packet_encoding.init_all()
         netcaps = get_network_caps()
         netif = {""    : bool(netifaces)}
         if netifaces_version:
@@ -536,9 +554,9 @@ def main():
                 print_nested_dict(ccaps)
         except Exception as e:
             print("No Crypto:")
-            print(" %s" % e)
+            print(f" {e}")
     return 0
 
 
-if __name__ == "__main__":
+if __name__ == "__main__":  # pragma: no cover
     main()
