@@ -161,6 +161,7 @@ class SocketProtocol:
         self._write_format_thread : Optional[Thread]= None        #started when needed
         self._source_has_more = Event()
         self.receive_pending = False
+        self.eof_pending = False
         self.wait_for_header = False
         self.source_has_more = self.source_has_more_start
         self.flush_then_close = self.do_flush_then_close
@@ -266,6 +267,7 @@ class SocketProtocol:
             "flush"                 : self.send_flush_flag,
             "has_more"              : shm and shm.is_set(),
             "receive-pending"       : self.receive_pending,
+            "eof-pending": self.eof_pending,
             }
         c = self.compressor
         if c:
@@ -766,13 +768,16 @@ class SocketProtocol:
         #add to the read queue (or whatever takes its place - see steal_connection)
         if not buf:
             eventlog("read thread: potential eof")
-            self.timeout_add(1000, self.check_eof, self.input_raw_packetcount)
+            if not self.eof_pending:
+                self.eof_pending = True
+                self.timeout_add(1000, self.check_eof, self.input_raw_packetcount)
         else:
             self._process_read(buf)
             self.input_raw_packetcount += 1
         return True
 
     def check_eof(self, raw_count=0) -> bool:
+        self.eof_pending = False
         if self.input_raw_packetcount <= raw_count:
             eventlog("check_eof: eof detected")
             # give time to the parse thread to call close itself,
