@@ -1,6 +1,6 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # This file is part of Xpra.
-# Copyright (C) 2011-2019 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2011-2023 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -8,7 +8,10 @@ import os
 import math
 import ctypes
 import struct
-import weakref
+from weakref import WeakValueDictionary
+from typing import Any, Callable, Dict, List, Type, Optional, Tuple
+
+from gi.repository import GLib      #@UnresolvedImport
 import objc                         #@UnresolvedImport
 import Quartz                       #@UnresolvedImport
 import Quartz.CoreGraphics as CG    #@UnresolvedImport
@@ -20,8 +23,8 @@ from Quartz.CoreGraphics import (
     CGDisplayRegisterReconfigurationCallback,   #@UnresolvedImport
     CGDisplayRemoveReconfigurationCallback,     #@UnresolvedImport
     )
-from AppKit import NSAppleEventManager, NSScreen, NSObject, NSBeep   #@UnresolvedImport
 from AppKit import (
+    NSObject, NSAppleEventManager, NSScreen, NSBeep,   #@UnresolvedImport
     NSApp, NSApplication, NSWorkspace,              #@UnresolvedImport
     NSWorkspaceActiveSpaceDidChangeNotification,    #@UnresolvedImport
     NSWorkspaceWillSleepNotification,               #@UnresolvedImport
@@ -32,9 +35,9 @@ from Foundation import (
     NSUserNotificationDefaultSoundName,             #@UnresolvedImport
     )
 
-from xpra.os_util import PYTHON2
 from xpra.util import envbool, envint, roundup
 from xpra.notifications.notifier_base import NotifierBase
+from xpra.platform.darwin import get_OSXApplication
 from xpra.log import Logger
 
 log = Logger("osx", "events")
@@ -47,8 +50,9 @@ SLEEP_HANDLER = envbool("XPRA_OSX_SLEEP_HANDLER", True)
 EVENT_LISTENER = envbool("XPRA_OSX_EVENT_LISTENER", True)
 OSX_WHEEL_MULTIPLIER = envint("XPRA_OSX_WHEEL_MULTIPLIER", 100)
 OSX_WHEEL_PRECISE_MULTIPLIER = envint("XPRA_OSX_WHEEL_PRECISE_MULTIPLIER", 1)
+OSX_WHEEL_DIVISOR = envint("XPRA_OSX_WHEEL_DIVISOR", 10)
 WHEEL = envbool("XPRA_WHEEL", True)
-NATIVE_NOTIFIER = envbool("XPRA_OSX_NATIVE_NOTIFIER", False)
+NATIVE_NOTIFIER = envbool("XPRA_OSX_NATIVE_NOTIFIER", True)
 SUBPROCESS_NOTIFIER = envbool("XPRA_OSX_SUBPROCESS_NOTIFIER", False)
 
 ALPHA = {
@@ -61,40 +65,17 @@ ALPHA = {
          CG.kCGImageAlphaNoneSkipFirst         : "SkipFirst",
    }
 #if there is an easier way of doing this, I couldn't find it:
+def nodbltime() -> float:
+    return -1
+GetDblTime : Callable = nodbltime
 try:
     Carbon_ctypes = ctypes.CDLL("/System/Library/Frameworks/Carbon.framework/Carbon")
     GetDblTime = Carbon_ctypes.GetDblTime
-except:
-    GetDblTime = None
+except Exception:
+    log("GetDblTime not found", exc_info=True)
 
 
-exit_cb = None
-def quit_handler(*_args):
-    global exit_cb
-    if exit_cb:
-        exit_cb()
-    else:
-        from xpra.gtk_common.quit import gtk_main_quit_really
-        gtk_main_quit_really()
-    return True
-
-def set_exit_cb(ecb):
-    global exit_cb
-    exit_cb = ecb
-
-
-macapp = None
-def get_OSXApplication():
-    global macapp
-    if macapp is None:
-        from xpra.gtk_common.gobject_compat import import_gtkosx_application
-        gtkosx_application = import_gtkosx_application()
-        macapp = gtkosx_application.Application()
-        macapp.connect("NSApplicationWillTerminate", quit_handler)
-    return macapp
-
-
-def do_init():
+def do_init() -> None:
     osxapp = get_OSXApplication()
     log("do_init() osxapp=%s", osxapp)
     if not osxapp:
@@ -110,10 +91,13 @@ def do_init():
     mh = getOSXMenuHelper(None)
     log("do_init() menu helper=%s", mh)
     osxapp.set_dock_menu(mh.build_dock_menu())
-    osxapp.set_menu_bar(mh.rebuild())
+    import warnings
+    with warnings.catch_warnings():
+        warnings.filterwarnings("ignore", message=".*invalid cast from 'GtkMenuBar'")
+        osxapp.set_menu_bar(mh.rebuild())
 
 
-def do_ready():
+def do_ready() -> None:
     osxapp = get_OSXApplication()
     if osxapp:
         log("%s()", osxapp.ready)
@@ -123,115 +107,107 @@ def do_ready():
 class OSX_Notifier(NotifierBase):
 
     def __init__(self, closed_cb=None, action_cb=None):
-        NotifierBase.__init__(self, closed_cb, action_cb)
+        super().__init__(closed_cb, action_cb)
+        self.gtk_notifier = None
+        self.gtk_notifications = set()
         self.notifications = {}
         self.notification_center = NSUserNotificationCenter.defaultUserNotificationCenter()
         assert self.notification_center
 
-    def show_notify(self, dbus_id, tray, nid, app_name, replaces_nid, app_icon, summary, body, actions, hints, expire_timeout, icon):
-        notification = NSUserNotification.alloc().init()
+    def show_notify(self, dbus_id, tray, nid, app_name, replaces_nid, app_icon,
+                    summary, body, actions, hints, expire_timeout, icon):
+        GTK_NOTIFIER = envbool("XPRA_OSX_GTK_NOTIFIER", True)
+        if actions and GTK_NOTIFIER:
+            #try to use GTK notifier if we have actions buttons to handle:
+            try:
+                from xpra.gtk_common.gtk_notifier import GTK_Notifier
+            except ImportError as e:
+                log("cannot use GTK notifier for handling actions: %s", e)
+            else:
+                self.gtk_notifier = GTK_Notifier(self.closed_cb, self.action_cb)
+                self.gtk_notifier.show_notify(dbus_id, tray, nid, app_name, replaces_nid, app_icon,
+                                              summary, body, actions, hints, expire_timeout, icon)
+                self.gtk_notifications.add(nid)
+                return
+        GLib.idle_add(self.do_show_notify, dbus_id, tray, nid, app_name, replaces_nid, app_icon, summary, body, actions, hints, expire_timeout, icon)
+
+    def do_show_notify(self, dbus_id, tray, nid:int, app_name, replaces_nid:int, app_icon,
+                       summary:str, body:str, actions, hints, expire_timeout:int, icon):
+        notification = NSUserNotification.alloc()
+        notification.init()
         notification.setTitle_(summary)
         notification.setInformativeText_(body)
         notification.setIdentifier_("%s" % nid)
         #enable sound:
         notification.setSoundName_(NSUserNotificationDefaultSoundName)
-        notifylog("show_notify(..) nid=%s, %s(%s)", nid, self.notification_center.deliverNotification_, notification)
+        notifylog("do_show_notify(..) nid=%s, %s(%s)", nid, self.notification_center.deliverNotification_, notification)
         self.notifications[nid] = notification
         self.notification_center.deliverNotification_(notification)
 
     def close_notify(self, nid):
-        notification = self.notifications.get(nid)
-        notifylog("close_notify(..) notification[%i]=%s", nid, notification)
-        if notification:
-            self.notification_center.removeDeliveredNotification_(notification)
+        try:
+            self.gtk_notifications.remove(nid)
+        except KeyError:
+            notification = self.notifications.get(nid)
+            notifylog("close_notify(..) notification[%i]=%s", nid, notification)
+            if notification:
+                GLib.idle_add(self.notification_center.removeDeliveredNotification_, notification)
+        else:
+            self.gtk_notifier.close_notify(nid)
 
     def cleanup(self):
-        NotifierBase.cleanup(self)
-        self.notification_center.removeAllDeliveredNotifications()
+        super().cleanup()
+        GLib.idle_add(self.notification_center.removeAllDeliveredNotifications)
 
 
-class OSX_Subprocess_Notifier(NotifierBase):
-    def show_notify(self, dbus_id, tray, nid, app_name, replaces_nid, app_icon, summary, body, actions, hints, expire_timeout, icon):
-        from xpra.platform.darwin import osx_notifier
-        osx_notifier_file = osx_notifier.__file__
-        if osx_notifier_file.endswith("pyc"):
-            osx_notifier_file = osx_notifier_file[:-1]
-        import time
-        #osx_notifier_file = "/Users/osx/osx_notifier.py"
-        from xpra.platform.paths import get_app_dir
-        base = get_app_dir()
-        #python_bin = "/usr/bin/python"
-        python_bin = os.path.join(base, "Resources", "bin", "python")
-        cmd = [python_bin, osx_notifier_file, "%s-%s" % (int(time.time()), nid), summary, body]
-        from xpra.child_reaper import getChildReaper
-        import subprocess
-        env = os.environ.copy()
-        for x in ("DYLD_LIBRARY_PATH", "XDG_CONFIG_DIRS", "XDG_DATA_DIRS",
-                  "GTK_DATA_PREFIX", "GTK_EXE_PREFIX", "GTK_PATH",
-                  "GTK2_RC_FILES", "GTK_IM_MODULE_FILE", "GDK_PIXBUF_MODULE_FILE",
-                  "PANGO_RC_FILE", "PANGO_LIBDIR", "PANGO_SYSCONFDIR",
-                  "CHARSETALIASDIR",
-                  "GST_BUNDLE_CONTENTS", "PYTHON", "PYTHONHOME",
-                  "PYTHONPATH"):
-            if x in env:
-                del env[x]
-        notifylog("running %s with env=%s", cmd, env)
-        proc = subprocess.Popen(cmd, env=env)
-        proc.wait()
-        notifylog("returned %i", proc.poll())
-        getChildReaper().add_process(proc, "notifier-%s" % nid, cmd, True, True)
-
-    def close_notify(self, nid):
-        pass
-
-
-def get_clipboard_native_class():
+def get_clipboard_native_class() -> str:
     return "xpra.platform.darwin.osx_clipboard.OSXClipboardProtocolHelper"
 
 
-def get_native_notifier_classes():
+def get_native_notifier_classes() -> List[Type]:
     v = []
     if NATIVE_NOTIFIER and NSUserNotificationCenter.defaultUserNotificationCenter():
         v.append(OSX_Notifier)
-    if SUBPROCESS_NOTIFIER:
-        v.append(OSX_Subprocess_Notifier)
     notifylog("get_native_notifier_classes()=%s", v)
     return v
 
-def get_native_tray_menu_helper_class():
+def get_native_tray_menu_helper_class() -> Optional[Callable]:
     if get_OSXApplication():
         from xpra.platform.darwin.osx_menu import getOSXMenuHelper
         return getOSXMenuHelper
     return None
 
-def get_native_tray_classes():
+def get_native_tray_classes() -> List[Type]:
     if get_OSXApplication():
         from xpra.platform.darwin.osx_tray import OSXTray
         return [OSXTray]
     return []
 
-def system_bell(*_args):
+def system_bell(*_args) -> bool:
     NSBeep()
     return True
 
 
-def _sizetotuple(s):
+def _sizetotuple(s) -> Tuple[int,int]:
     return int(s.width), int(s.height)
-def _recttotuple(r):
-    return tuple(int(v) for v in (r.origin.x, r.origin.y, r.size.width, r.size.height))
+def _recttotuple(r) -> Tuple[int,int,int,int]:
+    return int(r.origin.x), int(r.origin.y), int(r.size.width), int(r.size.height)
 
-def get_double_click_time():
+def get_double_click_time() -> int:
     try:
         #what are ticks? just an Apple retarded way of measuring elapsed time.
         #They must have considered gigaparsecs divided by teapot too, which is just as useful.
         #(but still call it "Time" you see)
         MS_PER_TICK = 1000.0/60
-        return int(GetDblTime() * MS_PER_TICK)
-    except:
-        return -1
+        v = GetDblTime()
+        if v>0:
+            return int(v * MS_PER_TICK)
+    except Exception:
+        pass
+    return -1
 
 
-def get_window_min_size():
+def get_window_min_size() -> Tuple[int,int]:
     #roughly enough to see the window buttons:
     return 120, 1
 
@@ -258,7 +234,7 @@ def get_window_frame_size(x, y, w, h):
                 "offset"    : (dx+dw//2, dy+dh),
                 "frame"     : (dx+dw//2, dx+dw//2, dy+dh, dy),
                 }
-    except:
+    except Exception:
         log("failed to query frame size using Quartz, using hardcoded value", exc_info=True)
         return {            #left, right, top, bottom:
                 "offset"    : (0, 22),
@@ -280,11 +256,7 @@ def get_workareas():
         frame = screen.frame()
         visibleFrame = screen.visibleFrame()
         log(" frame=%s, visibleFrame=%s", frame, visibleFrame)
-        try:
-            #10.7 onwards:
-            log(" backingScaleFactor=%s", screen.backingScaleFactor())
-        except:
-            pass
+        log(" backingScaleFactor=%s", screen.backingScaleFactor())
         x = int(visibleFrame.origin.x - frame.origin.x)
         y = int((frame.size.height - visibleFrame.size.height) - (visibleFrame.origin.y - frame.origin.y))
         w = int(visibleFrame.size.width)
@@ -293,7 +265,7 @@ def get_workareas():
     log("get_workareas()=%s", workareas)
     return workareas
 
-def get_vrefresh():
+def get_vrefresh() -> int:
     vrefresh = []
     try:
         err, active_displays, no = CG.CGGetActiveDisplayList(99, None, None)
@@ -313,7 +285,7 @@ def get_vrefresh():
     return -1
 
 
-def get_display_icc_info():
+def get_display_icc_info() -> Dict[Any,Dict]:
     info = {}
     try:
         err, active_displays, no = CG.CGGetActiveDisplayList(99, None, None)
@@ -335,8 +307,8 @@ def get_icc_info():
     return info
 
 
-def get_colorspace_info(cs):
-    MODELS = {
+def get_colorspace_info(cs) -> Dict[str,Any]:
+    MODELS : Dict[Any, str] = {
               CG.kCGColorSpaceModelUnknown     : "unknown",
               CG.kCGColorSpaceModelMonochrome  : "monochrome",
               CG.kCGColorSpaceModelRGB         : "RGB",
@@ -348,7 +320,7 @@ def get_colorspace_info(cs):
               }
     #base = CGColorSpaceGetBaseColorSpace(cs)
     #color_table = CGColorSpaceGetColorTable(cs)
-    def tomodelstr(v):
+    def tomodelstr(v) -> str:
         return MODELS.get(v, "unknown")
     defs = (
             ("name",                "CGColorSpaceCopyName",                 str),
@@ -362,7 +334,7 @@ def get_colorspace_info(cs):
             )
     return _call_CG_conv(defs, cs)
 
-def get_display_mode_info(mode):
+def get_display_mode_info(mode) -> Dict[str,Any]:
     defs = (
             ("width",               "CGDisplayModeGetWidth",                int),
             ("height",              "CGDisplayModeGetHeight",               int),
@@ -374,11 +346,11 @@ def get_display_mode_info(mode):
             )
     return _call_CG_conv(defs, mode)
 
-def get_display_modes_info(modes):
+def get_display_modes_info(modes) -> Dict[int,Any]:
     return dict((i,get_display_mode_info(mode)) for i,mode in enumerate(modes))
 
 
-def _call_CG_conv(defs, argument):
+def _call_CG_conv(defs:Tuple[Tuple[str,str,Callable],...], argument) -> Dict[str,Any]:
     #utility for calling functions on CG with an argument,
     #then convert the return value using another function
     #missing functions are ignored, and None values are skipped
@@ -400,7 +372,7 @@ def _call_CG_conv(defs, argument):
             log("function %s does not exist", fn_name)
     return info
 
-def get_display_info(did):
+def get_display_info(did) -> Dict[str,Any]:
     defs = (
             ("height",                  "CGDisplayPixelsHigh",              int),
             ("width",                   "CGDisplayPixelsWide",              int),
@@ -435,9 +407,9 @@ def get_display_info(did):
         log("failed to query display modes: %s", e)
     return info
 
-def get_displays_info():
+def get_displays_info() -> Dict[str,Any]:
     did = CG.CGMainDisplayID()
-    info = {
+    info : Dict[str,Any] = {
             "main" : get_display_info(did),
             }
     err, active_displays, no = CG.CGGetActiveDisplayList(99, None, None)
@@ -450,38 +422,40 @@ def get_displays_info():
             info.setdefault("online", {})[i] = get_display_info(odid)
     return info
 
-def get_info():
+def get_info() -> Dict[str,Any]:
     from xpra.platform.gui import get_info_base
     i = get_info_base()
     try:
         i["displays"] = get_displays_info()
-    except:
+    except Exception:
         log.error("Error: OSX get_display_info failed", exc_info=True)
     return i
 
 
 #keep track of the window object for each view
-VIEW_TO_WINDOW = weakref.WeakValueDictionary()
+VIEW_TO_WINDOW : WeakValueDictionary[int,Any] = WeakValueDictionary()
 
-def add_window_hooks(window):
-    if WHEEL and PYTHON2:
-        global VIEW_TO_WINDOW
-        try:
-            gdkwin = window.get_window()
-            VIEW_TO_WINDOW[gdkwin.nsview] = window
-        except:
-            log("add_window_hooks(%s)", window, exc_info=True)
-            log.error("Error: failed to associate window %s with its nsview", window, exc_info=True)
+def get_nsview(window) -> int:
+    try:
+        from xpra.platform.darwin.gdk3_bindings import get_nsview_ptr
+    except ImportError:
+        return 0
+    try:
+        return get_nsview_ptr(window.get_window())
+    except Exception:
+        return 0
 
-def remove_window_hooks(window):
-    if WHEEL and PYTHON2:
-        global VIEW_TO_WINDOW
-        #this should be redundant as we use weak references:
-        try:
-            gdkwin = window.get_window()
-            del VIEW_TO_WINDOW[gdkwin.nsview]
-        except:
-            pass
+def add_window_hooks(window) -> None:
+    if WHEEL:
+        nsview = get_nsview(window)
+        if nsview:
+            VIEW_TO_WINDOW[nsview] = window
+
+def remove_window_hooks(window) -> None:
+    if WHEEL:
+        nsview = get_nsview(window)
+        if nsview:
+            VIEW_TO_WINDOW.pop(nsview, None)
 
 
 def get_CG_imagewrapper(rect=None):
@@ -510,7 +484,7 @@ def get_CG_imagewrapper(rect=None):
     argb = CG.CGDataProviderCopyData(prov)
     return ImageWrapper(x, y, width, height, argb, "BGRX", 24, rowstride)
 
-def take_screenshot():
+def take_screenshot() -> Tuple[int,int,str,int,bytes]:
     log("grabbing screenshot")
     from PIL import Image                       #@UnresolvedImport
     from io import BytesIO
@@ -524,19 +498,9 @@ def take_screenshot():
     buf.close()
     return w, h, "png", image.get_rowstride(), data
 
-def force_focus(duration=2000):
+def force_focus(duration=2000) -> None:
     enable_focus_workaround()
-    from xpra.gtk_common.gobject_compat import import_glib
-    glib = import_glib()
-    glib.timeout_add(duration, disable_focus_workaround)
-
-
-def show_with_focus_workaround(show_cb):
-    from xpra.gtk_common.gobject_compat import import_glib
-    glib = import_glib()
-    enable_focus_workaround()
-    show_cb()
-    glib.timeout_add(500, disable_focus_workaround)
+    GLib.timeout_add(duration, disable_focus_workaround)
 
 
 __osx_open_signal = False
@@ -544,23 +508,24 @@ __osx_open_signal = False
 #to get the NSApplicationOpenFile signal,
 #or the GURLHandler.
 OPEN_SIGNAL_WAIT = envint("XPRA_OSX_OPEN_SIGNAL_WAIT", 500)
-def add_open_handlers(open_file_cb, open_url_cb):
-    assert open_file_cb and open_url_cb
+def add_open_handlers(open_file_cb:Callable, open_url_cb:Callable) -> None:
 
     def idle_add(fn, *args):
-        from xpra.gtk_common.gobject_compat import import_glib
-        glib = import_glib()
-        glib.idle_add(fn, *args)
+        GLib.idle_add(fn, *args)
 
-    def open_URL(url):
+    def open_URL(url) -> bool:
         global __osx_open_signal
+        if __osx_open_signal:
+            return True
         __osx_open_signal = True
         log("open_URL(%s)", url)
         idle_add(open_url_cb, url)
         return True
 
-    def open_file(_, filename, *args):
+    def open_file(_, filename:str, *args) -> bool:
         global __osx_open_signal
+        if __osx_open_signal:
+            return True
         __osx_open_signal = True
         log("open_file(%s, %s)", filename, args)
         idle_add(open_file_cb, filename)
@@ -568,29 +533,29 @@ def add_open_handlers(open_file_cb, open_url_cb):
     register_file_handler(open_file)
     register_URL_handler(open_URL)
 
-def wait_for_open_handlers(show_cb, open_file_cb, open_url_cb, delay=OPEN_SIGNAL_WAIT):
-    assert show_cb and open_file_cb and open_url_cb
-    from xpra.gtk_common.gobject_compat import import_glib
-    glib = import_glib()
-
+def wait_for_open_handlers(show_cb:Callable,
+                           open_file_cb:Callable,
+                           open_url_cb:Callable,
+                           delay:int=OPEN_SIGNAL_WAIT) -> None:
     add_open_handlers(open_file_cb, open_url_cb)
     def may_show():
         global __osx_open_signal
         log("may_show() osx open signal=%s", __osx_open_signal)
         if not __osx_open_signal:
+            __osx_open_signal = True
             force_focus()
             show_cb()
-    glib.timeout_add(delay, may_show)
+    GLib.timeout_add(delay, may_show)
 
-def register_file_handler(handler):
+def register_file_handler(handler:Callable) -> None:
     log("register_file_handler(%s)", handler)
     try:
         get_OSXApplication().connect("NSApplicationOpenFile", handler)
     except Exception as e:
         log.error("Error: cannot handle file associations:")
-        log.error(" %s", e)
+        log.estr(e)
 
-def register_URL_handler(handler):
+def register_URL_handler(handler:Callable) -> None:
     log("register_URL_handler(%s)", handler)
     class GURLHandler(NSObject):
         def handleEvent_withReplyEvent_(self, event, reply_event):
@@ -613,15 +578,16 @@ def register_URL_handler(handler):
         )
 
 
-class Delegate(NSObject):
-    def applicationDidFinishLaunching_(self, notification):
-        log("applicationDidFinishLaunching_(%s)", notification)
-        if SLEEP_HANDLER:
-            self.register_sleep_handlers()
-        if WHEEL and PYTHON2:
-            from xpra.platform.darwin.gdk_bindings import init_quartz_filter, set_wheel_event_handler   #@UnresolvedImport
-            set_wheel_event_handler(self.wheel_event_handler)
-            init_quartz_filter()
+class AppDelegate(NSObject):
+
+    def init(self) -> None:
+        self = objc.super(AppDelegate, self).init()
+        if self is None:
+            return None
+        self.callbacks : Dict[str,Callable] = {}
+        self.workspace = None
+        self.notificationCenter = None
+        return self
 
     @objc.python_method
     def wheel_event_handler(self, nsview, deltax, deltay, precise):
@@ -637,7 +603,7 @@ class Delegate(NSObject):
                 m = OSX_WHEEL_PRECISE_MULTIPLIER
             else:
                 m = OSX_WHEEL_MULTIPLIER
-            v = abs(distance)/10.0*m
+            v = m*abs(distance)/OSX_WHEEL_DIVISOR
             if v>1:
                 #cancel out some of the crazy fast scroll acceleration from macos:
                 v = math.sqrt(v)
@@ -650,14 +616,16 @@ class Delegate(NSObject):
         dy = normalize_precision(deltay)
         if dx!=0 or dy!=0:
             client = window._client
-            wid = window._id
-            client.wheel_event(wid, dx, dy)
+            wid = window.wid
+            pointer = window.get_mouse_position()
+            device_id = -1
+            client.wheel_event(device_id, wid, dx, dy, pointer)
         return True
 
     @objc.python_method
     def register_sleep_handlers(self):
         log("register_sleep_handlers()")
-        self.workspace          = NSWorkspace.sharedWorkspace()
+        self.workspace : NSWorkspace = NSWorkspace.sharedWorkspace()
         self.notificationCenter = self.workspace.notificationCenter()
         def add_observer(fn, val):
             self.notificationCenter.addObserver_selector_name_object_(self, fn, val, None)
@@ -666,12 +634,14 @@ class Delegate(NSObject):
         add_observer(self.receiveWakeNotification_, NSWorkspaceDidWakeNotification)
         add_observer(self.receiveWorkspaceChangeNotification_, NSWorkspaceActiveSpaceDidChangeNotification)
 
-    @objc.signature(b'B@:#B')
+
+    @objc.typedSelector(b'B@:#B')
     def applicationShouldHandleReopen_hasVisibleWindows_(self, ns_app, flag):
         log("applicationShouldHandleReopen_hasVisibleWindows%s", (ns_app, flag))
-        self.delegate_cb("deiconify_callback")
+        self.delegate_cb("deiconify")
         return True
 
+    @objc.typedSelector(b'v@:I')
     def receiveWorkspaceChangeNotification_(self, aNotification):
         workspacelog("receiveWorkspaceChangeNotification_(%s)", aNotification)
         if not CGWindowListCopyWindowInfo:
@@ -688,38 +658,39 @@ class Delegate(NSObject):
                     our_windows[num] = name
             workspacelog("workspace change - our windows on screen: %s", our_windows)
             if our_windows:
-                self.delegate_cb("wake_callback")
+                self.delegate_cb("wake")
             else:
-                self.delegate_cb("sleep_callback")
-        except:
+                self.delegate_cb("sleep")
+        except Exception:
             workspacelog.error("Error querying workspace info", exc_info=True)
 
     #def application_openFile_(self, application, fileName):
     #    log.warn("application_openFile_(%s, %s)", application, fileName)
-
+    @objc.typedSelector(b'v@:I')
     def receiveSleepNotification_(self, aNotification):
         log("receiveSleepNotification_(%s) sleep_callback=%s", aNotification, self.sleep_callback)
-        self.delegate_cb("sleep_callback")
+        self.delegate_cb("sleep")
 
+    @objc.typedSelector(b'v@:I')
     def receiveWakeNotification_(self, aNotification):
         log("receiveWakeNotification_(%s)", aNotification)
-        self.delegate_cb("wake_callback")
+        self.delegate_cb("wake")
 
     @objc.python_method
     def delegate_cb(self, name):
         #find the named callback and call it
-        callback = getattr(self, name, None)
+        callback = self.callbacks.get(name)
         log("delegate_cb(%s)=%s", name, callback)
         if callback:
             try:
                 callback()
-            except:
+            except Exception:
                 log.error("Error in %s callback %s", name, callback, exc_info=True)
 
 
-def disable_focus_workaround():
+def disable_focus_workaround() -> None:
     NSApp.activateIgnoringOtherApps_(False)
-def enable_focus_workaround():
+def enable_focus_workaround() -> None:
     NSApp.activateIgnoringOtherApps_(True)
 
 
@@ -728,29 +699,35 @@ def can_access_display() -> bool:
     d = Quartz.CGSessionCopyCurrentDictionary()
     if not d:
         return False
-    if d.get("kCGSSessionOnConsoleKey", 0)==0:
+    kCGSSessionOnConsoleKey = d.get("kCGSSessionOnConsoleKey", 0)
+    log("kCGSSessionOnConsoleKey=%s", kCGSSessionOnConsoleKey)
+    if kCGSSessionOnConsoleKey==0:
         #GUI session doesn't own the console, or the console's screens are asleep
         return False
-    if d.get("CGSSessionScreenIsLocked", 0):
+    CGSSessionScreenIsLocked = d.get("CGSSessionScreenIsLocked", 0)
+    log("CGSSessionScreenIsLocked=%s", CGSSessionScreenIsLocked)
+    if CGSSessionScreenIsLocked:
         #screen is locked
         return False
     return True
 
 
-class ClientExtras(object):
+class ClientExtras:
     def __init__(self, client=None, opts=None):
         if OSX_FOCUS_WORKAROUND and client:
             def first_ui_received(*_args):
                 enable_focus_workaround()
                 client.timeout_add(OSX_FOCUS_WORKAROUND, disable_focus_workaround)
             client.connect("first-ui-received", first_ui_received)
-        swap_keys = opts and opts.swap_keys
-        log("ClientExtras.__init__(%s, %s) swap_keys=%s", client, opts, swap_keys)
+        log("ClientExtras.__init__(%s, %s)", client, opts)
         self.client = client
         self.event_loop_started = False
         self.check_display_timer = 0
         self.display_is_asleep = False
+        self.shared_app = None
+        self.delegate = None
         if opts and client:
+            swap_keys = opts.swap_keys
             log("setting swap_keys=%s using %s", swap_keys, client.keyboard_helper)
             if client.keyboard_helper and client.keyboard_helper.keyboard:
                 log("%s.swap_keys=%s", client.keyboard_helper.keyboard, swap_keys)
@@ -758,7 +735,7 @@ class ClientExtras(object):
         if client:
             self.check_display_timer = client.timeout_add(60*1000, self.check_display)
 
-    def cleanup(self):
+    def cleanup(self) -> None:
         cdt = self.check_display_timer
         client = self.client
         if cdt and client:
@@ -775,35 +752,32 @@ class ClientExtras(object):
             log("failed to unregister display reconfiguration callback")
         self.client = None
 
-    def ready(self):
+    def ready(self) -> None:
         if EVENT_LISTENER:
             try:
                 self.setup_event_listener()
-            except:
+            except Exception:
                 log.error("Error setting up OSX event listener", exc_info=True)
 
-    def setup_event_listener(self):
-        log("setup_event_listener()")
-        if NSObject is object:
-            log.warn("NSObject is missing, not setting up OSX event listener")
-            return
-        self.shared_app = None
-        self.delegate = None
+    def setup_event_listener(self) -> None:
         self.shared_app = NSApplication.sharedApplication()
-
-        self.delegate = Delegate.alloc().init()
+        log(f"setup_event_listener() delegate={self.delegate}, shared app={self.shared_app}")
+        self.delegate = AppDelegate.alloc()
+        self.delegate.init()
         self.delegate.retain()
         if self.client:
-            self.delegate.sleep_callback = self.client.suspend
-            self.delegate.wake_callback = self.client.resume
-            self.delegate.deiconify_callback = self.client.deiconify_windows
+            self.delegate.callbacks.update({
+                "sleep" : self.client.suspend,
+                "wake" : self.client.resume,
+                "deiconify" : self.client.deiconify_windows,
+                })
         self.shared_app.setDelegate_(self.delegate)
         log("setup_event_listener() the application delegate has been registered")
         r = CGDisplayRegisterReconfigurationCallback(self.display_change, self)
         if r!=0:
             log.warn("Warning: failed to register display reconfiguration callback")
 
-    def display_change(self, display, flags, userinfo):
+    def display_change(self, display, flags, userinfo) -> None:
         log("display_change%s", (display, flags, userinfo))
         c = self.client
         #The display mode has changed
@@ -811,7 +785,7 @@ class ClientExtras(object):
         if (flags & kCGDisplaySetModeFlag) and c and c.opengl_enabled:
             c.reinit_windows()
 
-    def check_display(self):
+    def check_display(self) -> bool:
         log("check_display()")
         try:
             c = self.client
@@ -836,7 +810,7 @@ class ClientExtras(object):
             self.check_display_timer = 0
             return False
 
-    def run(self):
+    def run(self) -> None:
         #this is for running standalone
         log("starting console event loop")
         self.event_loop_started = True
@@ -845,7 +819,7 @@ class ClientExtras(object):
         #when running from the GTK main loop, we rely on another part of the code
         #to run the event loop for us
 
-    def stop(self):
+    def stop(self) -> None:
         if self.event_loop_started:
             self.event_loop_started = False
             import PyObjCTools.AppHelper as AppHelper   #@UnresolvedImport
@@ -857,6 +831,8 @@ def main():
     with program_context("OSX Extras"):
         log.enable_debug()
         ce = ClientExtras(None, None)
+        ce.check_display()
+        ce.ready()
         ce.run()
 
 
