@@ -1,10 +1,11 @@
 # This file is part of Xpra.
-# Copyright (C) 2011-2018 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2011-2023 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import sys
 import os.path
+from typing import List
 
 from xpra.util import envbool, get_util_logger
 
@@ -23,16 +24,12 @@ def do_get_resources_dir():
     #FUGLY warning: importing gtkosx_application causes the dock to appear,
     #and in some cases we don't want that.. so use the env var XPRA_SKIP_UI as workaround for such cases:
     if not envbool("XPRA_SKIP_UI", False):
+        from xpra.platform.darwin import get_OSXApplication
+        macapp = get_OSXApplication()
         try:
-            from xpra.gtk_common.gobject_compat import import_gtkosx_application
-            gtkosx_application = import_gtkosx_application()
-            try:
-                rsc = gtkosx_application.gtkosx_application_get_resource_path()
-                debug("get_resources_dir() gtkosx_application_get_resource_path=%s", rsc)
-            except:
-                #maybe we're not running from an app bundle?
-                pass
-        except:
+            rsc = macapp.get_resource_path()
+            debug("get_resources_dir() %s.get_resource_path=%s", macapp, rsc)
+        except Exception:
             global _gtkosx_warning_
             if _gtkosx_warning_ is False:
                 _gtkosx_warning_ = True
@@ -55,7 +52,7 @@ def do_get_resources_dir():
     debug("get_resources_dir()=%s", rsc)
     return rsc
 
-def do_get_app_dir():
+def do_get_app_dir() -> str:
     from xpra.platform.paths import get_resources_dir
     rsc = get_resources_dir()
     CONTENTS = "/Contents/"
@@ -65,21 +62,21 @@ def do_get_app_dir():
     debug("get_app_dir()=%s", rsc)
     return rsc  #hope for the best..
 
-def do_get_icon_dir():
+def do_get_icon_dir() -> str:
     from xpra.platform.paths import get_resources_dir
     i = os.path.join(get_resources_dir(), "share", "xpra", "icons")
     debug("get_icon_dir()=%s", i)
     return i
 
 
-def do_get_default_conf_dirs():
+def do_get_default_conf_dirs() -> List[str]:
     #the default config file we install into the Resources folder:
     #ie: /Volumes/Xpra/Xpra.app/Contents/Resources/etc
     from xpra.platform.paths import get_resources_dir
     return [os.path.join(get_resources_dir(), "etc", "xpra")]
 
-def do_get_system_conf_dirs():
-    #the system wide configuration directory
+def do_get_system_conf_dirs() -> List[str]:
+    #the system-wide configuration directory
     dirs = []
     try:
         from Foundation import (
@@ -91,52 +88,67 @@ def do_get_system_conf_dirs():
         for x in sdirs:
             #ie: "/Library/Application Support/Xpra"
             dirs.append(os.path.join(x, "Xpra"))
-    except:
+    except Exception:
         #fallback to hardcoded:
         default_conf_dir = "/Library/Application Support/Xpra"
         dirs = [os.environ.get("XPRA_SYSCONF_DIR", default_conf_dir)]
     dirs.append("/etc/xpra")
     return dirs
 
-def do_get_ssh_conf_dirs():
+def do_get_ssh_conf_dirs() -> List[str]:
     return ["/private/etc", "/etc/ssh", "/usr/local/etc/ssh", "~/.ssh", "~/ssh"]
 
-def do_get_user_conf_dirs(_uid):
+def do_get_user_conf_dirs(_uid) -> List[str]:
     #the system wide configuration directory
     dirs = []
     try:
         #when running sandboxed, it may look like this:
         #~/Library/Containers/<bundle_id>/Data/Library/Application Support/
-        from Foundation import  NSSearchPathForDirectoriesInDomains, NSApplicationSupportDirectory, NSUserDomainMask    #@UnresolvedImport
+        from Foundation import (
+            NSSearchPathForDirectoriesInDomains,                #@UnresolvedImport
+            NSApplicationSupportDirectory, NSUserDomainMask,    #@UnresolvedImport
+            )
         udirs = NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory, NSUserDomainMask, False)
         for x in udirs:
             dirs.append(os.path.join(x, "Xpra"))
-    except:
+    except Exception:
         #fallback to hardcoded:
-        dirs = ["/Library/Application Support/Xpra"]
+        from os import getuid
+        if getuid() == 0:
+            dirs = ["/Library/Application Support/Xpra"]
     dirs.append("~/.xpra")
     return dirs
 
-def do_get_default_log_dirs():
+def do_get_default_log_dirs() -> List[str]:
     dirs = []
     try:
         from Foundation import (
-            NSSearchPathForDirectoriesInDomains, NSLibraryDirectory, NSUserDomainMask,  #@UnresolvedImport
+            NSSearchPathForDirectoriesInDomains,    #@UnresolvedImport
+            NSLibraryDirectory, NSUserDomainMask,   #@UnresolvedImport
             )
         udirs = NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, False)
         for x in udirs:
             #ie: ~/Library/
             dirs.append(os.path.join(x, "Logs", "Xpra"))
-    except:
+    except Exception:
         dirs.append("~/Library/Logs/Xpra")
-    dirs.append("/tmp")
+    import tempfile
+    dirs.append(tempfile.gettempdir())
     return dirs
 
-def do_get_socket_dirs():
-    return ["/var/tmp/%s-Xpra" % os.getuid(), "~/.xpra"]
+def do_get_sessions_dir() -> str:
+    return "~/.xpra/sessions"
+
+def do_get_socket_dirs() -> List[str]:
+    #return ["/var/tmp/%s-Xpra" % os.getuid(), "~/.xpra"]
+    import tempfile
+    return [tempfile.gettempdir(), "~/.xpra"]
+
+def do_get_client_socket_dirs() -> List[str]:
+    return ["~/.xpra/clients"]
 
 
-def do_get_download_dir():
+def do_get_download_dir() -> str:
     d = "~/Downloads"
     try:
         from Foundation import (
@@ -144,36 +156,29 @@ def do_get_download_dir():
             )
         d = NSSearchPathForDirectoriesInDomains(NSDownloadsDirectory, NSUserDomainMask, False)[0]
         #(should be "~/Downloads")
-    except:
+    except Exception:
         pass
     if not os.path.exists(os.path.expanduser(d)):
         return "~"
     return d
 
 
-def do_get_desktop_background_paths():
+def do_get_desktop_background_paths() -> List[str]:
     return [
         "/System/Library/CoreServices/DefaultDesktop.jpg",
         "/Library/Desktop Pictures/*jpg",
         ]
 
 
-def do_get_libexec_dir():
-    from xpra.platform.paths import get_app_dir
-    base = get_app_dir()
-    p = os.path.join(base, "Helpers")
-    return p
-
-
-def do_get_sshpass_command():
+def do_get_sshpass_command() -> str:
     from xpra.platform.paths import get_app_dir
     base = get_app_dir()
     p = os.path.join(base, "Resources", "bin", "sshpass")
     if os.path.exists(p):
         return p
-    return None
+    return ""
 
-def do_get_xpra_command():
+def do_get_xpra_command() -> List[str]:
     if sys.argv and sys.argv[0].lower().endswith("/xpra"):
         return [sys.argv[0]]
     #try to use the one from the app bundle:
@@ -184,17 +189,24 @@ def do_get_xpra_command():
         return [xpra_cmd]
     return ["xpra"]
 
-def do_get_nodock_command():
-    #try to use the subapp:
+
+def _get_helpers_dir() -> str:
+    from xpra.platform.paths import get_app_dir
+    base = get_app_dir()
+    p = os.path.join(base, "Helpers")
+    return p
+
+def do_get_nodock_command() -> List[str]:
+    # try to use the sub-app:
     from xpra.platform.paths import get_app_dir
     base = get_app_dir()
     subapp = os.path.join(base, "Xpra_NoDock.app", "Contents")
     if os.path.exists(subapp) and os.path.isdir(subapp):
         base = subapp
-    #appstore builds have script wrappers:
+    # AppStore builds have script wrappers:
     helper = os.path.join(base, "Resources", "scripts", "Xpra")
     if not os.path.exists(helper):
-        helper = os.path.join(base, "Helpers", "Xpra")
+        helper = os.path.join(_get_helpers_dir(), "Xpra")
     if not os.path.exists(helper):
         #having a dock is still better than
         #trying to run a command that does not exist!
@@ -202,5 +214,12 @@ def do_get_nodock_command():
         return get_xpra_command()
     return [helper]
 
-def do_get_sound_command():
+def do_get_audio_command() -> List[str]:
     return do_get_nodock_command()
+
+
+def do_get_python_exec_command() -> List[str]:
+    return [os.path.join(_get_helpers_dir(), "Python"), "-c"]
+
+def do_get_python_execfile_command() -> List[str]:
+    return [os.path.join(_get_helpers_dir(), "Python")]
