@@ -126,6 +126,8 @@ EXTENSIONS : Tuple[str, ...] = ("png", "svg", "xpm")
 
 
 def check_xdg() -> bool:
+    if not ENABLED:
+        return False
     try:
         # pylint: disable=import-outside-toplevel
         from xdg.Menu import Menu, MenuEntry
@@ -158,7 +160,7 @@ def load_entry_icon(props:Dict):
     for x in ("Exec", "TryExec"):
         cmd = props.get(x)
         if cmd and not cmd.endswith(os.path.sep):
-            cmd = os.path.basename(cmd)
+            cmd = os.path.basename(cmd).split(" ")[0]
             if cmd not in names:
                 names.append(cmd)
     filename = find_icon(*names)
@@ -232,10 +234,10 @@ def find_theme_icon(*names):
 
 def find_glob_icon(*names, category:str="categories"):
     if not LOAD_GLOB:
-        return None
+        return ""
     icondirs = getattr(IconTheme, "icondirs", [])
     if not icondirs:
-        return None
+        return ""
     dirnames = (category, )
     pathnames = []
     for name in names:
@@ -254,8 +256,8 @@ def find_glob_icon(*names, category:str="categories"):
                 v = icon_util.load_icon_from_file(f)
                 if v:
                     log(f"found icon for {names} with glob {pathname!r}: {f}")
-                    return v
-    return None
+                    return f
+    return ""
 
 
 def noicondata(d:Dict) -> Dict:
@@ -351,7 +353,7 @@ def remove_icons(menu_data):
     return filt
 
 def load_menu():
-    if not ENABLED or not check_xdg():
+    if not check_xdg():
         return {}
     icon_util.large_icons.clear()
     start = monotonic()
@@ -373,15 +375,10 @@ def load_menu():
     return xdg_menu_data
 
 def load_xdg_menu_data():
-    try:
-        from xdg.Menu import parse, Menu  #pylint: disable=import-outside-toplevel
-    except ImportError:
-        log("load_xdg_menu_data()", exc_info=True)
-        if first_time("no-python-xdg"):
-            log.warn("Warning: cannot use application menu data:")
-            log.warn(" no python-xdg module")
-        return None
+    if not check_xdg():
+        return {}
     menu = None
+    from xdg.Menu import parse, Menu  # pylint: disable=import-outside-toplevel
     error = None
     #see ticket #2340,
     #invalid values for XDG_CONFIG_DIRS can cause problems,
@@ -483,7 +480,7 @@ def load_xdg_menu_data():
 
 def load_applications(menu_data=None):
     entries : Dict[str,Any] = {}
-    if not LOAD_APPLICATIONS:
+    if not LOAD_APPLICATIONS or not check_xdg():
         return entries
     def already_has_name(name):
         if not menu_data:
