@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2010 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2011-2020 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2011-2023 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -8,8 +8,10 @@ import ctypes
 from ctypes.wintypes import HANDLE
 from ctypes import create_string_buffer, byref
 from ctypes.wintypes import DWORD
+from typing import List, Dict, Tuple, Optional, Callable
 
 from xpra.platform.win32.common import (
+    ActivateKeyboardLayout,
     GetKeyState, GetKeyboardLayoutList, GetKeyboardLayout,
     GetIntSystemParametersInfo, GetKeyboardLayoutName,
     GetWindowThreadProcessId,
@@ -25,7 +27,7 @@ from xpra.log import Logger
 log = Logger("keyboard")
 
 
-def _GetKeyboardLayoutList():
+def _GetKeyboardLayoutList() -> List[int]:
     max_items = 32
     #PHANDLE = ctypes.POINTER(HANDLE)
     handle_list = (HANDLE*max_items)()
@@ -36,6 +38,36 @@ def _GetKeyboardLayoutList():
         layouts.append(int(handle_list[i]))
     return layouts
 
+def x11_layouts_to_win32_hkl() -> Dict[str,int]:
+    KMASKS = {
+        0xffffffff : (0, 16),
+        0xffff  : (0, ),
+        0x3ff   : (0, ),
+        }
+    layout_to_hkl = {}
+    max_items = 32
+    try:
+        handle_list = (HANDLE*max_items)()
+        count = GetKeyboardLayoutList(max_items, ctypes.byref(handle_list))
+        for i in range(count):
+            hkl = handle_list[i]
+            hkli = int(hkl)
+            for mask, bitshifts in KMASKS.items():
+                kbid = 0
+                for bitshift in bitshifts:
+                    kbid = (hkli & mask)>>bitshift
+                    if kbid in WIN32_LAYOUTS:
+                        break
+                if kbid in WIN32_LAYOUTS:
+                    code, _, _, _, _layout, _variants = WIN32_LAYOUTS.get(kbid)
+                    log("found keyboard layout '%s' / %#x with variants=%s, code '%s' for kbid=%#x",
+                        _layout, kbid, _variants, code, hkli)
+                    if _layout and _layout not in layout_to_hkl:
+                        layout_to_hkl[_layout] = hkli
+                        break
+    except Exception:
+        log("x11_layouts_to_win32_hkl()", exc_info=True)
+    return layout_to_hkl
 
 EMULATE_ALTGR = envbool("XPRA_EMULATE_ALTGR", True)
 EMULATE_ALTGR_CONTROL_KEY_DELAY = envint("XPRA_EMULATE_ALTGR_CONTROL_KEY_DELAY", 50)
@@ -48,8 +80,8 @@ class Keyboard(KeyboardBase):
         * simulate 'Alt_Gr'
     """
 
-    def __init__(self):
-        KeyboardBase.__init__(self)
+    def init_vars(self) -> None:
+        super().init_vars()
         self.num_lock_modifier = None
         self.altgr_modifier = None
         self.delayed_event = None
@@ -60,11 +92,22 @@ class Keyboard(KeyboardBase):
         #workaround for "fr" keyboards, which use a different key name under X11:
         KEY_TRANSLATIONS[("dead_tilde", 65107,  50)]    = "asciitilde"
         KEY_TRANSLATIONS[("dead_grave", 65104,  55)]    = "grave"
+        self.__x11_layouts_to_win32_hkl = x11_layouts_to_win32_hkl()
+
+    def set_platform_layout(self, layout:str) -> None:
+        hkl = self.__x11_layouts_to_win32_hkl.get(layout)
+        if hkl is None:
+            log(f"asked layout ({layout}) has no corresponding registered keyboard handle")
+            return
+        # https://learn.microsoft.com/en-us/windows/win32/api/winuser/nf-winuser-activatekeyboardlayout
+        # KLF_SETFORPROCESS|KLF_REORDER = 0x108
+        if not ActivateKeyboardLayout(hkl, 0x108):
+            log(f"ActivateKeyboardLayout: cannot change layout to {layout}")
 
     def __repr__(self):
         return "win32.Keyboard"
 
-    def set_modifier_mappings(self, mappings):
+    def set_modifier_mappings(self, mappings) -> None:
         super().set_modifier_mappings(mappings)
         self.num_lock_modifier = self.modifier_keys.get("Num_Lock")
         log("set_modifier_mappings found 'Num_Lock' with modifier value: %s", self.num_lock_modifier)
@@ -75,7 +118,7 @@ class Keyboard(KeyboardBase):
                 log("set_modifier_mappings found 'AltGr'='%s' with modifier value: %s", x, self.altgr_modifier)
                 break
 
-    def mask_to_names(self, mask):
+    def mask_to_names(self, mask) -> List[str]:
         """ Patch NUMLOCK and AltGr """
         names = super().mask_to_names(mask)
         if EMULATE_ALTGR:
@@ -92,7 +135,7 @@ class Keyboard(KeyboardBase):
                     names.remove(self.num_lock_modifier)
                 log("mask_to_names(%s) GetKeyState(VK_NUMLOCK)=%s, names=%s", mask, numlock, names)
             except Exception:
-                pass
+                log("mask_to_names error modifying numlock", exc_info=True)
         else:
             log("mask_to_names(%s)=%s", mask, names)
         return names
@@ -121,11 +164,13 @@ class Keyboard(KeyboardBase):
         return  {}, [], ["lock"]
 
 
-    def get_all_x11_layouts(self):
+    def get_all_x11_layouts(self) -> Dict[str,str]:
         x11_layouts = {}
         for win32_layout in WIN32_LAYOUTS.values():
             #("ARA", "Saudi Arabia",   "Arabic",                   1356,   "ar", []),
             x11_layout = win32_layout[4]
+            if not x11_layout:
+                continue
             if x11_layout in x11_layouts:
                 continue
             name = win32_layout[2]
@@ -133,10 +178,14 @@ class Keyboard(KeyboardBase):
         return x11_layouts
 
 
-    def get_layout_spec(self):
-        KMASKS = (0xffffffff, 0xffff, 0x3ff)
+    def get_layout_spec(self) -> Tuple[str,List[str],str,List[str],str]:
+        KMASKS = {
+            0xffffffff : (0, 16),
+            0xffff  : (0, ),
+            0x3ff   : (0, ),
+            }
         layout = None
-        layouts = []
+        layouts_defs = {}
         variant = None
         variants = None
         options = ""
@@ -145,43 +194,25 @@ class Keyboard(KeyboardBase):
             l = _GetKeyboardLayoutList()
             log("GetKeyboardLayoutList()=%s", csv(hex(v) for v in l))
             for hkl in l:
-                for mask in KMASKS:
-                    kbid = hkl & mask
+                for mask, bitshifts in KMASKS.items():
+                    kbid = 0
+                    for bitshift in bitshifts:
+                        kbid = (hkl & mask)>>bitshift
+                        if kbid in WIN32_LAYOUTS:
+                            break
                     if kbid in WIN32_LAYOUTS:
                         code, _, _, _, _layout, _variants = WIN32_LAYOUTS.get(kbid)
-                        log("found keyboard layout '%s' / %#x with variants=%s, code '%s' for kbid=%i (%#x)",
-                            _layout, kbid, _variants, code, kbid, hkl)
-                        if _layout not in layouts:
-                            layouts.append(_layout)
-                            if not layout_code:
-                                layout_code = kbid
+                        log("found keyboard layout '%s' / %#x with variants=%s, code '%s' for kbid=%#x",
+                            _layout, kbid, _variants, code, hkl)
+                        if _layout and _layout not in layouts_defs:
+                            layouts_defs[_layout] = hkl
                             break
         except Exception as e:
-            log.error("Error: failed to detect keyboard layouts:")
-            log.error(" %s", e)
-
-        try:
-            pid = DWORD(0)
-            GetWindowThreadProcessId(0, byref(pid))
-            tid = GetWindowThreadProcessId(0, pid)
-            hkl = GetKeyboardLayout(tid)
-            log("GetKeyboardLayout(%i)=%#x", tid, hkl)
-            for mask in KMASKS:
-                kbid = hkl & mask
-                if kbid not in WIN32_LAYOUTS:
-                    continue
-                code, _, _, _, layout0, variants = WIN32_LAYOUTS.get(kbid)
-                log("found keyboard layout '%s' / %#x with variants=%s, code '%s' for kbid=%i (%#x)",
-                    layout0, kbid, variants, code, kbid, hkl)
-                if layout0 not in layouts:
-                    layouts.append(layout0)
-                #only override "layout" if unset:
-                if not layout and layout0:
-                    layout = layout0
-                    layout_code = kbid
-        except Exception:
-            log.error("Error: failed to detect keyboard layout", exc_info=True)
+            log("get_layout_spec()", exc_info=True)
+            log.error("Error: failed to detect keyboard layouts using GetKeyboardLayoutList:")
+            log.estr(e)
 
+        descr = None
         KL_NAMELENGTH = 9
         name_buf = create_string_buffer(KL_NAMELENGTH)
         if GetKeyboardLayoutName(name_buf):
@@ -199,31 +230,60 @@ class Keyboard(KeyboardBase):
                     kbdef = WIN32_KEYBOARDS.get(val)
                     log("get_layout_spec() WIN32_KEYBOARDS[%#x]=%s", val, kbdef)
                     if kbdef:
-                        _layout, descr = kbdef
+                        _layout, _descr = kbdef
                         if _layout=="??":
                             log.warn("Warning: the X11 codename for %#x is not known", val)
-                            log.warn(" only identified as '%s'", descr)
+                            log.warn(" only identified as '%s'", _descr)
                             log.warn(" please file a bug report")
                             continue
-                        elif not layout:
+                        if _layout not in layouts_defs:
+                            layouts_defs[_layout] = ival
+                        if not layout:
                             layout = _layout
-                            layout_code = val
+                            descr = _descr
+                            layout_code = ival
                             break
                 if not layout:
                     log.warn("Warning: unknown keyboard layout %#x", ival)
                     log.warn(" please file a bug report")
                     self.last_layout_message = layout
 
+        try:
+            pid = DWORD(0)
+            GetWindowThreadProcessId(0, byref(pid))
+            tid = GetWindowThreadProcessId(0, pid)
+            hkl = GetKeyboardLayout(tid)
+            log("GetKeyboardLayout(%i)=%#x", tid, hkl)
+            for mask in KMASKS:
+                kbid = hkl & mask
+                if kbid not in WIN32_LAYOUTS:
+                    continue
+                code, _, _, _, layout0, variants = WIN32_LAYOUTS.get(kbid)
+                log("found keyboard layout '%s' / %#x with variants=%s, code '%s' for kbid=%i (%#x)",
+                    layout0, kbid, variants, code, kbid, hkl)
+                if layout0 not in layouts_defs:
+                    layouts_defs[layout0] = hkl
+                #only override "layout" if unset:
+                if not layout and layout0:
+                    layout = layout0
+                    layout_code = hkl
+        except Exception:
+            log.error("Error: failed to detect keyboard layout using GetKeyboardLayout", exc_info=True)
+
+        layouts = list(layouts_defs.keys())
         if layouts and not layout:
             layout = layouts[0]
+            layout_code = layouts_defs.get(layout, 0)
 
         if layout and self.last_layout_message!=layout:
-            log.info("keyboard layout code %#x", layout_code)
-            log.info("identified as '%s' : %s", descr, layout)
+            if descr:
+                log.info(f"keyboard layout {descr!r} : {layout!r} ({layout_code:#x})")
+            else:
+                log.info(f"keyboard layout {layout!r} ({layout_code:#x})")
             self.last_layout_message = layout
-        return layout,layouts,variant,variants, options
+        return layout, layouts, variant, variants, options
 
-    def get_keyboard_repeat(self):
+    def get_keyboard_repeat(self) -> Optional[Tuple[int,int]]:
         try:
             _delay = GetIntSystemParametersInfo(win32con.SPI_GETKEYBOARDDELAY)
             _speed = GetIntSystemParametersInfo(win32con.SPI_GETKEYBOARDSPEED)
@@ -234,13 +294,13 @@ class Keyboard(KeyboardBase):
             _speed = min(31, max(0, _speed))
             speed = int(1000/(2.5+27.5*_speed/31))
             log("keyboard repeat speed(%s)=%s, delay(%s)=%s", _speed, speed, _delay, delay)
-            return  delay,speed
+            return delay,speed
         except Exception as e:
             log.error("failed to get keyboard rate: %s", e)
         return None
 
 
-    def process_key_event(self, send_key_action_cb, wid, key_event):
+    def process_key_event(self, send_key_action_cb:Callable, wid:int, key_event) -> None:
         """ Caps_Lock and Num_Lock don't work properly: they get reported more than once,
             they are reported as not pressed when the key is down, etc
             So we just ignore those and rely on the list of "modifiers" passed
@@ -258,23 +318,19 @@ class Keyboard(KeyboardBase):
         if EMULATE_ALTGR and self.altgr_modifier and len(self.modifier_mappings)>0:
             rmenu = GetKeyState(win32con.VK_RMENU)
             if key_event.keyname=="Control_L":
-                log("process_key_event: %s pressed=%s, with GetKeyState(VK_RMENU)=%s", key_event.keyname, key_event.pressed, rmenu)
-                #AltGr is often preceded by a spurious "Control_L" event
+                log("process_key_event: %s pressed=%s, with GetKeyState(VK_RMENU)=%s",
+                    key_event.keyname, key_event.pressed, rmenu)
+                #AltGr key events are often preceded by a spurious "Control_L" event
                 #delay this one a little bit so we can skip it if an "AltGr" does come through next:
-                if key_event.pressed:
-                    if rmenu in (0, 1):
-                        self.delayed_event = (send_key_action_cb, wid, key_event)
-                        #needed for altgr emulation timeouts:
-                        from gi.repository import GLib
-                        GLib.timeout_add(EMULATE_ALTGR_CONTROL_KEY_DELAY, self.send_delayed_key)
-                    return
-                if not key_event.pressed and rmenu not in (0, 1):
-                    #unpressed: could just skip it?
-                    #(but maybe the real one got pressed.. and this would get it stuck)
-                    pass
+                if rmenu in (0, 1):
+                    self.delayed_event = (send_key_action_cb, wid, key_event)
+                    #needed for altgr emulation timeouts:
+                    from gi.repository import GLib  # @UnresolvedImport
+                    GLib.timeout_add(EMULATE_ALTGR_CONTROL_KEY_DELAY, self.send_delayed_key)
+                return
             if key_event.keyname=="Alt_R":
                 log("process_key_event: Alt_R pressed=%s, with GetKeyState(VK_RMENU)=%s", key_event.pressed, rmenu)
-                if rmenu in (0, 1) and key_event.pressed:
+                if rmenu in (0, 1):
                     #cancel "Control_L" if one was due:
                     self.delayed_event = None
                 #modify the key event so that it will only trigger the modifier update,
@@ -288,7 +344,7 @@ class Keyboard(KeyboardBase):
         self.send_delayed_key()
         super().process_key_event(send_key_action_cb, wid, key_event)
 
-    def send_delayed_key(self):
+    def send_delayed_key(self) -> None:
         #timeout: this must be a real one, send it now
         dk = self.delayed_event
         log("send_delayed_key() delayed_event=%s", dk)
