@@ -10,15 +10,18 @@ import tempfile
 import platform
 from ctypes import (
     WinDLL,  # @UnresolvedImport
-    create_unicode_buffer
-    )
-from ctypes.wintypes import MAX_PATH
+    create_unicode_buffer,
+    HRESULT
+)
+from ctypes.wintypes import MAX_PATH, HWND, INT, HANDLE, DWORD, LPWSTR
 from typing import List
 
 from xpra.os_util import get_util_logger
 
 shell32 = WinDLL("shell32", use_last_error=True)
 SHGetFolderPath = shell32.SHGetFolderPathW
+SHGetFolderPath.argtypes = [HWND, INT, HANDLE, DWORD, LPWSTR]
+SHGetFolderPath.restype = HRESULT
 
 CSIDL_APPDATA = 26
 CSIDL_LOCAL_APPDATA = 28
@@ -50,12 +53,17 @@ def get_appdata_dir(roaming=True) -> str:
         # UAC in vista onwards will not allow us to write where the software is installed,
         # so we place the log file (etc) in "~/Application Data"
         appdata = os.environ.get("APPDATA" if roaming else "LOCALAPPDATA", "")
+    if appdata and not os.path.exists(appdata):
+        try:
+            os.mkdir(appdata)
+        except OSError as e:
+            get_util_logger().warn("Warning: %r does not exist and cannot be created:", appdata)
+            get_util_logger().warn(" %s", e)
+            appdata = ""
     if not appdata:
         #we need some kind of path..
         appdata = tempfile.gettempdir()
         assert appdata, "cannot find any usable directory for log files"
-    if not os.path.exists(appdata):
-        os.mkdir(appdata)
     data_dir = os.path.join(appdata, "Xpra")
     return data_dir
 
@@ -77,7 +85,7 @@ def do_get_icon_dir() -> str:
     return os.path.join(get_resources_dir(), "icons")
 
 def do_get_default_log_dirs() -> List[str]:
-    dd = _get_data_dir()
+    dd = _get_data_dir(False)
     temp = tempfile.gettempdir()
     if dd==temp:
         return [temp]
@@ -147,7 +155,7 @@ def do_get_default_conf_dirs() -> List[str]:
     return [os.path.join(get_app_dir(), "etc", "xpra")]
 
 def do_get_user_conf_dirs(_uid) -> List[str]:
-    dd = _get_data_dir()
+    dd = _get_data_dir(True)
     #ie: "C:\Users\<user name>\AppData\Roaming"
     SYSTEMROOT = os.environ.get("SYSTEMROOT", "")
     #ie: when running as a system service, we may get:
