@@ -1,27 +1,27 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2012-2019 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2012-2023 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os
-from collections import namedtuple
-from ctypes import (
-    create_string_buffer, create_unicode_buffer,
-    sizeof, byref, addressof, c_int,
-    )
-from ctypes.wintypes import RECT, POINT, BYTE, MAX_PATH
+import re
+from time import monotonic
+from typing import Dict, Any, Tuple, Optional, Type, List
+from ctypes import create_unicode_buffer, sizeof, byref, c_ulong
+from ctypes.wintypes import RECT, POINT, BYTE
 
-from xpra.os_util import strtobytes
-from xpra.util import envbool, prettify_plug_name, csv, XPRA_APP_ID, XPRA_IDLE_NOTIFICATION_ID
+from xpra.util import envbool, prettify_plug_name, csv, XPRA_APP_ID, NotificationID
 from xpra.scripts.config import InitException
 from xpra.server.gtk_server_base import GTKServerBase
-from xpra.server.shadow.gtk_shadow_server_base import GTKShadowServerBase, MULTI_WINDOW
+from xpra.server.shadow.gtk_root_window_model import GTKImageCapture
+from xpra.server.shadow.gtk_shadow_server_base import GTKShadowServerBase
 from xpra.server.shadow.root_window_model import RootWindowModel
 from xpra.platform.win32 import constants as win32con
 from xpra.platform.win32.gui import get_desktop_name, get_fixed_cursor_size
 from xpra.platform.win32.keyboard_config import KeyboardConfig, fake_key
 from xpra.platform.win32.win32_events import get_win32_event_listener, POWER_EVENTS
+from xpra.platform.win32.shadow_cursor import get_cursor_data
 from xpra.platform.win32.gdi_screen_capture import GDICapture
 from xpra.log import Logger
 
@@ -33,10 +33,6 @@ from xpra.platform.win32.common import (
     GetWindowThreadProcessId,
     GetSystemMetrics,
     SetPhysicalCursorPos, GetPhysicalCursorPos, GetCursorInfo, CURSORINFO,
-    GetDC, CreateCompatibleDC, CreateCompatibleBitmap, SelectObject, DeleteObject,
-    ReleaseDC, DeleteDC, DrawIconEx, GetBitmapBits,
-    GetIconInfo, ICONINFO, Bitmap, GetIconInfoExW, ICONINFOEXW,
-    GetObjectA,
     GetKeyboardState, SetKeyboardState,
     EnumDisplayMonitors, GetMonitorInfo,
     mouse_event,
@@ -61,20 +57,28 @@ BUTTON_EVENTS = {
                  (4, False) : NOEVENT,
                  (5, True)  : (win32con.MOUSEEVENTF_WHEEL,      -win32con.WHEEL_DELTA),
                  (5, False) : NOEVENT,
+                 (6, True)  : (win32con.MOUSEEVENTF_HWHEEL,     win32con.WHEEL_DELTA),
+                 (6, False) : NOEVENT,
+                 (7, True)  : (win32con.MOUSEEVENTF_HWHEEL,     -win32con.WHEEL_DELTA),
+                 (7, False) : NOEVENT,
+                 (8, True)  : (win32con.MOUSEEVENTF_XDOWN,      win32con.XBUTTON1),
+                 (8, False) : (win32con.MOUSEEVENTF_XUP,        win32con.XBUTTON1),
+                 (9, True)  : (win32con.MOUSEEVENTF_XDOWN,      win32con.XBUTTON2),
+                 (9, False) : (win32con.MOUSEEVENTF_XUP,        win32con.XBUTTON2),
                  }
 
 SEAMLESS = envbool("XPRA_WIN32_SEAMLESS", False)
-SHADOW_NVFBC = envbool("XPRA_SHADOW_NVFBC", True)
-SHADOW_GDI = envbool("XPRA_SHADOW_GDI", True)
-NVFBC_CUDA = envbool("XPRA_NVFBC_CUDA", True)
+NVFBC = envbool("XPRA_SHADOW_NVFBC", True)
+GDI = envbool("XPRA_SHADOW_GDI", True)
+GSTREAMER = envbool("XPRA_SHADOW_GSTREAMER", True)
 
 
-def get_root_window_size():
+def get_root_window_size() -> Tuple[int,int]:
     w = GetSystemMetrics(win32con.SM_CXVIRTUALSCREEN)
     h = GetSystemMetrics(win32con.SM_CYVIRTUALSCREEN)
     return w, h
 
-def get_monitors():
+def get_monitors() -> List[Dict[str,Any]]:
     monitors = []
     for m in EnumDisplayMonitors():
         mi = GetMonitorInfo(m)
@@ -82,172 +86,75 @@ def get_monitors():
     return monitors
 
 
-def get_cursor_data(hCursor):
-    #w, h = get_fixed_cursor_size()
-    if not hCursor:
-        return None
-    x, y = 0, 0
-    dc = None
-    memdc = None
-    bitmap = None
-    old_handle = None
-    pixels = None
-    try:
-        ii = ICONINFO()
-        ii.cbSize = sizeof(ICONINFO)
-        if not GetIconInfo(hCursor, byref(ii)):
-            raise WindowsError()    #@UndefinedVariable
-        x = ii.xHotspot
-        y = ii.yHotspot
-        cursorlog("get_cursor_data(%#x) hotspot at %ix%i, hbmColor=%#x, hbmMask=%#x",
-                  hCursor, x, y, ii.hbmColor or 0, ii.hbmMask or 0)
-        if not ii.hbmColor:
-            #FIXME: we don't handle black and white cursors
-            return None
-        iie = ICONINFOEXW()
-        iie.cbSize = sizeof(ICONINFOEXW)
-        if not GetIconInfoExW(hCursor, byref(iie)):
-            raise WindowsError()    #@UndefinedVariable
-        name = iie.szResName[:MAX_PATH]
-        cursorlog("wResID=%#x, sxModName=%s, szResName=%s", iie.wResID, iie.sxModName[:MAX_PATH], name)
-        bm = Bitmap()
-        if not GetObjectA(ii.hbmColor, sizeof(Bitmap), byref(bm)):
-            raise WindowsError()    #@UndefinedVariable
-        cursorlog("cursor bitmap: type=%i, width=%i, height=%i, width bytes=%i, planes=%i, bits pixel=%i, bits=%#x",
-                  bm.bmType, bm.bmWidth, bm.bmHeight, bm.bmWidthBytes, bm.bmPlanes, bm.bmBitsPixel, bm.bmBits or 0)
-        w = bm.bmWidth
-        h = bm.bmHeight
-        dc = GetDC(None)
-        assert dc, "failed to get a drawing context"
-        memdc = CreateCompatibleDC(dc)
-        assert memdc, "failed to get a compatible drawing context from %s" % dc
-        bitmap = CreateCompatibleBitmap(dc, w, h)
-        assert bitmap, "failed to get a compatible bitmap from %s" % dc
-        old_handle = SelectObject(memdc, bitmap)
-
-        #check if icon is animated:
-        UINT_MAX = 2**32-1
-        if not DrawIconEx(memdc, 0, 0, hCursor, w, h, UINT_MAX, 0, 0):
-            cursorlog("cursor is animated!")
-
-        #if not DrawIcon(memdc, 0, 0, hCursor):
-        if not DrawIconEx(memdc, 0, 0, hCursor, w, h, 0, 0, win32con.DI_NORMAL):
-            raise WindowsError()    #@UndefinedVariable
-
-        buf_size = bm.bmWidthBytes*h
-        buf = create_string_buffer(b"", buf_size)
-        r = GetBitmapBits(bitmap, buf_size, byref(buf))
-        cursorlog("get_cursor_data(%#x) GetBitmapBits(%#x, %#x, %#x)=%i", hCursor, bitmap, buf_size, addressof(buf), r)
-        if not r:
-            cursorlog.error("Error: failed to copy screen bitmap data")
-            return None
-        elif r!=buf_size:
-            cursorlog.warn("Warning: invalid cursor buffer size, got %i bytes but expected %i", r, buf_size)
-            return None
-        else:
-            #32-bit data:
-            pixels = bytearray(strtobytes(buf.raw))
-            has_alpha = False
-            has_pixels = False
-            for i in range(len(pixels)//4):
-                has_pixels = has_pixels or pixels[i*4]!=0 or pixels[i*4+1]!=0 or pixels[i*4+2]!=0
-                has_alpha = has_alpha or pixels[i*4+3]!=0
-                if has_pixels and has_alpha:
-                    break
-            if has_pixels and not has_alpha:
-                #generate missing alpha - don't ask me why
-                for i in range(len(pixels)//4):
-                    if pixels[i*4]!=0 or pixels[i*4+1]!=0 or pixels[i*4+2]!=0:
-                        pixels[i*4+3] = 0xff
-        return [0, 0, w, h, x, y, hCursor, bytes(pixels), strtobytes(name)]
-    except Exception as e:
-        cursorlog("get_cursor_data(%#x)", hCursor, exc_info=True)
-        cursorlog.error("Error: failed to grab cursor:")
-        cursorlog.error(" %s", str(e) or type(e))
-        return None
-    finally:
-        if old_handle:
-            SelectObject(memdc, old_handle)
-        if bitmap:
-            DeleteObject(bitmap)
-        if memdc:
-            DeleteDC(memdc)
-        if dc:
-            ReleaseDC(None, dc)
-
-
 def init_capture(w, h, pixel_depth=32):
-    capture = None
-    if SHADOW_NVFBC:
+    if NVFBC:
         try:
-            from xpra.codecs.nvfbc.fbc_capture_win import init_nvfbc_library
+            from xpra.codecs.nvidia.nvfbc.capture import get_capture_instance
+            capture = get_capture_instance()
         except ImportError:
             log("NvFBC capture is not available", exc_info=True)
         else:
             try:
-                if init_nvfbc_library():
-                    log("NVFBC_CUDA=%s", NVFBC_CUDA)
-                    pixel_format = {
-                        24  : "RGB",
-                        32  : "BGRX",
-                        30  : "r210",
-                        }[pixel_depth]
-                    if NVFBC_CUDA:
-                        from xpra.codecs.nvfbc.fbc_capture_win import NvFBC_CUDACapture #@UnresolvedImport
-                        capture = NvFBC_CUDACapture()
-                    else:
-                        from xpra.codecs.nvfbc.fbc_capture_win import NvFBC_SysCapture  #@UnresolvedImport
-                        capture = NvFBC_SysCapture()
-                    capture.init_context(w, h, pixel_format)
+                pixel_format = {
+                    24  : "RGB",
+                    32  : "BGRX",
+                    30  : "r210",
+                    }[pixel_depth]
+                capture.init_context(w, h, pixel_format)
+                #this will test the capture and ensure we can call get_image()
+                capture.refresh()
+                return capture
             except Exception as e:
-                capture = None
                 log("NvFBC_Capture", exc_info=True)
                 log.warn("Warning: NvFBC screen capture initialization failed:")
-                log.warn(" %s", e)
-                log.warn(" using the slower GDI capture code")
-                del e
-    if not capture:
-        if SHADOW_GDI:
-            capture = GDICapture()
-        if not capture:
-            raise Exception("no screen capture methods enabled (GDI capture is disabled)")
-    log("init_capture()=%s", capture)
-    return capture
+                for x in str(e).replace(". ", ":").split(":"):
+                    if x.strip() and x!="nvfbc":
+                        log.warn(" %s", x.strip())
+    if GSTREAMER:
+        try:
+            from xpra.codecs.gstreamer.capture import Capture
+        except ImportError:
+            log("no GStreamer capture", exc_info=True)
+        else:
+            capture_elements = [
+                "d3d11screencapturesrc",
+                "dx9screencapsrc",
+                #"gdiscreencapsrc",
+                ]
+            for el in capture_elements:
+                log(f"testing gstreamer capture using {el}")
+                try:
+                    capture = Capture(el, pixel_format="BGR", width=w, height=h)
+                    capture.start()
+                    image = capture.get_image(0, 0, w, h)
+                    if image:
+                        log(f"using gstreamer element {el}")
+                        return capture
+                except Exception:
+                    log(f"gstreamer failed to capture the screen using {el}", exc_info=True)
+    if GDI:
+        return GDICapture()
+    return GTKImageCapture(None)
 
 
 class SeamlessRootWindowModel(RootWindowModel):
 
-    def __init__(self, root, capture):
-        RootWindowModel.__init__(self, root, capture)
+    def __init__(self, root, capture, title, geometry):
+        super().__init__(root, capture, title, geometry)
         log("SeamlessRootWindowModel(%s, %s) SEAMLESS=%s", root, capture, SEAMLESS)
         self.property_names.append("shape")
         self.dynamic_property_names.append("shape")
         self.rectangles = self.get_shape_rectangles(logit=True)
-        self.shape_notify = []
 
-    def refresh_shape(self):
+    def refresh_shape(self) -> None:
         rectangles = self.get_shape_rectangles()
         if rectangles==self.rectangles:
             return  #unchanged
         self.rectangles = rectangles
         shapelog("refresh_shape() sending notify for updated rectangles: %s", rectangles)
-        #notify listeners:
-        PSpec = namedtuple("PSpec", "name")
-        pspec = PSpec(name="shape")
-        for cb, args in self.shape_notify:
-            shapelog("refresh_shape() notifying: %s", cb)
-            try:
-                cb(self, pspec, *args)
-            except Exception:
-                shapelog.error("error in shape notify callback %s", cb, exc_info=True)
+        self.notify("shape")
 
-    def connect(self, signal, cb, *args):
-        if signal=="notify::shape":
-            self.shape_notify.append((cb, args))
-        else:
-            RootWindowModel.connect(self, signal, cb, *args)
-
-    def get_shape_rectangles(self, logit=False):
+    def get_shape_rectangles(self, logit=False) -> List:
         #get the list of windows
         l = log
         if logit or envbool("XPRA_SHAPE_DEBUG", False):
@@ -261,7 +168,7 @@ class SeamlessRootWindowModel(RootWindowModel):
             if not IsWindowVisible(hwnd):
                 l("skipped invisible window %#x", hwnd)
                 return True
-            pid = c_int()
+            pid = c_ulong()
             thread_id = GetWindowThreadProcessId(hwnd, byref(pid))
             if pid==ourpid:
                 l("skipped our own window %#x", hwnd)
@@ -278,7 +185,7 @@ class SeamlessRootWindowModel(RootWindowModel):
             if GetWindowRect(hwnd, byref(rect))==0:
                 l("GetWindowRect failure")
                 return True
-            left, top, right, bottom = rect.left, rect.top, rect.right, rect.bottom
+            left, top, right, bottom = int(rect.left), int(rect.top), int(rect.right), int(rect.bottom)
             if right<0 or bottom<0:
                 l("skipped offscreen window at %ix%i", right, bottom)
                 return True
@@ -324,7 +231,7 @@ class SeamlessRootWindowModel(RootWindowModel):
                 h = bottom
             rectangles.append((left, top, w, h))
             return True
-        EnumWindows(EnumWindowsProc(enum_windows_cb))
+        EnumWindows(EnumWindowsProc(enum_windows_cb), 0)
         l("get_shape_rectangles()=%s", rectangles)
         return sorted(rectangles)
 
@@ -333,23 +240,36 @@ class SeamlessRootWindowModel(RootWindowModel):
             shape = {"Bounding.rectangles" : self.rectangles}
             #provide clip rectangle? (based on workspace area?)
             return shape
-        return RootWindowModel.get_property(self, prop)
+        return super().get_property(prop)
+
+
+class Win32ShadowModel(RootWindowModel):
+    __slots__ = ("hwnd", "iconic")
+    def __init__(self, root_window, capture=None, title="", geometry=None):
+        super().__init__(root_window, capture, title, geometry)
+        self.hwnd = 0
+        self.iconic = geometry[2]==-32000 and geometry[3]==-32000
+        self.property_names.append("hwnd")
+        self.dynamic_property_names.append("size-hints")
+
+    def get_id(self) -> int:
+        return self.hwnd
+
+    def __repr__(self):
+        return "Win32ShadowModel(%s : %24s : %s)" % (self.capture, self.geometry, self.hwnd)
 
 
 class ShadowServer(GTKShadowServerBase):
 
-    def __init__(self):
-        GTKShadowServerBase.__init__(self)
-        self.keycodes = {}
+    def __init__(self, display=None, multi_window=True):
+        super().__init__(multi_window)
+        self.pixel_depth = 32
         self.cursor_handle = None
         self.cursor_data = None
+        self.cursor_errors = [0.0, 0]
         if GetSystemMetrics(win32con.SM_SAMEDISPLAYFORMAT)==0:
             raise InitException("all the monitors must use the same display format")
         el = get_win32_event_listener()
-        from xpra.net.bytestreams import set_continue_wait
-        #on win32, we want to wait just a little while,
-        #to prevent servers spinning wildly on non-blocking sockets:
-        set_continue_wait(5)
         #TODO: deal with those messages?
         el.add_event_callback(win32con.WM_POWERBROADCAST,   self.power_broadcast_event)
         #el.add_event_callback(WM_WTSSESSION_CHANGE,         self.session_change_event)
@@ -361,42 +281,42 @@ class ShadowServer(GTKShadowServerBase):
                                  win32con.WM_INPUTLANGCHANGE    : "WM_INPUTLANGCHANGE",
                                  win32con.WM_WININICHANGE       : "WM_WININICHANGE",
                                  })
-        #non-blocking server sockets (TCP and named pipes):
-        from xpra.net.bytestreams import CONTINUE_ERRNO
-        import errno
-        CONTINUE_ERRNO[errno.WSAEWOULDBLOCK] = "WSAEWOULDBLOCK"     #@UndefinedVariable pylint: disable=no-member
 
-    def init(self, opts):
+    def init(self, opts) -> None:
         self.pixel_depth = int(opts.pixel_depth) or 32
         if self.pixel_depth not in (24, 30, 32):
             raise InitException("unsupported pixel depth: %s" % self.pixel_depth)
-        GTKShadowServerBase.init(self, opts)
+        super().init(opts)
 
 
-    def power_broadcast_event(self, wParam, lParam):
+    def power_broadcast_event(self, wParam, lParam) -> None:
         log("WM_POWERBROADCAST: %s/%s", POWER_EVENTS.get(wParam, wParam), lParam)
         if wParam==win32con.PBT_APMSUSPEND:
             log.info("WM_POWERBROADCAST: PBT_APMSUSPEND")
             for source in self._server_sources.values():
-                source.may_notify(XPRA_IDLE_NOTIFICATION_ID, "Server Suspending",
+                source.may_notify(NotificationID.IDLE, "Server Suspending",
                                   "This Xpra server is going to suspend,\nthe connection is likely to be interrupted soon.",
                                   expire_timeout=10*1000, icon_name="shutdown")
         elif wParam==win32con.PBT_APMRESUMEAUTOMATIC:
             log.info("WM_POWERBROADCAST: PBT_APMRESUMEAUTOMATIC")
 
 
-    def guess_session_name(self, _procs=None):
+    def guess_session_name(self, _procs=None) -> None:
         desktop_name = get_desktop_name()
         log("get_desktop_name()=%s", desktop_name)
         if desktop_name:
             self.session_name = desktop_name
 
 
-    def print_screen_info(self):
-        w, h = self.get_root_window_size()
+    def print_screen_info(self) -> None:
+        size = self.get_root_window_size()
+        if not size:
+            # we probably don't have access to the screen
+            return
+        w, h = size
         try:
             display = prettify_plug_name(self.root.get_screen().get_display().get_name())
-        except:
+        except Exception:
             display = ""
         self.do_print_screen_info(display, w, h)
 
@@ -411,40 +331,103 @@ class ShadowServer(GTKShadowServerBase):
         w, h = get_root_window_size()
         return init_capture(w, h, self.pixel_depth)
 
-    def makeRootWindowModels(self):
-        log("makeRootWindowModels() root=%s", self.root)
-        self.capture = self.setup_capture()
+
+    def get_root_window_model_class(self) -> Type:
         if SEAMLESS:
-            model_class = SeamlessRootWindowModel
-        else:
-            model_class = RootWindowModel
-        if not MULTI_WINDOW:
-            return (model_class(self.root, self.capture),)
+            return SeamlessRootWindowModel
+        return RootWindowModel
+
+
+    def makeDynamicWindowModels(self):
+        assert self.window_matches
+        ourpid = os.getpid()
+        taskbar = FindWindowA("Shell_TrayWnd", None)
+        windows = {}
+        def enum_windows_cb(hwnd, lparam):
+            if not IsWindowVisible(hwnd):
+                log("window %#x is not visible", hwnd)
+                return True
+            pid = c_ulong()
+            thread_id = GetWindowThreadProcessId(hwnd, byref(pid))
+            if pid==ourpid:
+                log("skipped our own window %#x, thread id=%#x", hwnd, thread_id)
+                return True
+            rect = RECT()
+            if GetWindowRect(hwnd, byref(rect))==0:
+                log("GetWindowRect failure")
+                return True
+            if hwnd==taskbar:
+                log("skipped taskbar")
+                return True
+            #skipping IsWindowEnabled check
+            length = GetWindowTextLengthW(hwnd)
+            buf = create_unicode_buffer(length+1)
+            if GetWindowTextW(hwnd, buf, length+1)>0:
+                window_title = buf.value
+            else:
+                window_title = ''
+            left, top, right, bottom = int(rect.left), int(rect.top), int(rect.right), int(rect.bottom)
+            w = right-left
+            h = bottom-top
+            if left<=-32000 or top<=-32000:
+                log("%r is not visible: %s", window_title, (left, top, w, h))
+            if w<=0 and h<=0:
+                log("skipped invalid window size: %ix%i", w, h)
+                return True
+            windows[hwnd] = (window_title, (left, top, w, h))
+            return True
+        EnumWindows(EnumWindowsProc(enum_windows_cb), 0)
+        log("makeDynamicWindowModels() windows=%s", windows)
         models = []
-        monitors = get_monitors()
-        for i, monitor in enumerate(monitors):
-            geom = monitor["Monitor"]
-            x1, y1, x2, y2 = geom
-            assert x1<x2 and y1<y2
-            model = model_class(self.root, self.capture)
-            model.title = monitor["Device"].lstrip("\\\\.\\")
-            model.geometry = x1, y1, x2-x1, y2-y1
-            screenlog("monitor %i: %10s geometry=%s (from %s)", i, model.title, model.geometry, geom)
+        def add_model(hwnd, title, geometry):
+            model = Win32ShadowModel(self.root, self.capture, title=title, geometry=geometry)
+            model.hwnd = hwnd
             models.append(model)
-            screenlog("makeRootWindowModels: model(%s)=%s", monitor, model)
-        log("makeRootWindowModels()=%s", models)
+        for m in self.window_matches:
+            window = None
+            try:
+                if m.startswith("0x"):
+                    hwnd = int(m, 16)
+                else:
+                    hwnd = int(m)
+                if hwnd:
+                    window = windows.pop(hwnd, None)
+                    if window:
+                        add_model(hwnd, *window)
+            except ValueError:
+                namere = re.compile(m, re.IGNORECASE)
+                for hwnd, window in tuple(windows.items()):
+                    title, geometry = window
+                    if namere.match(title):
+                        add_model(hwnd, title, geometry)
+                        windows.pop(hwnd)
+        log("makeDynamicWindowModels()=%s", models)
         return models
 
 
-    def refresh(self):
-        v = GTKShadowServerBase.refresh(self)
+    def get_shadow_monitors(self) -> List:
+        #convert to the format expected by GTKShadowServerBase:
+        monitors = []
+        for i, monitor in enumerate(get_monitors()):
+            geom = monitor["Monitor"]
+            x1, y1, x2, y2 = geom
+            assert x1<x2 and y1<y2
+            plug_name = monitor["Device"].lstrip("\\\\.\\")
+            monitors.append((plug_name, x1, y1, x2-x1, y2-y1, 1))
+            screenlog("monitor %i: %10s coordinates: %s", i, plug_name, geom)
+        log("get_shadow_monitors()=%s", monitors)
+        return monitors
+
+
+    def refresh(self) -> bool:
+        v = super().refresh()
         if v and SEAMLESS:
             for rwm in self._id_to_window.values():
                 rwm.refresh_shape()
         log("refresh()=%s", v)
         return v
 
-    def do_get_cursor_data(self):
+    def do_get_cursor_data(self) -> Optional[Tuple]:
         ci = CURSORINFO()
         ci.cbSize = sizeof(CURSORINFO)
         GetCursorInfo(byref(ci))
@@ -469,23 +452,37 @@ class ShadowServer(GTKShadowServerBase):
             ((w,h), [(w,h), ]),
             )
 
-    def get_pointer_position(self):
+    def get_pointer_position(self) -> Tuple[int,int]:
         pos = POINT()
         GetPhysicalCursorPos(byref(pos))
         return pos.x, pos.y
 
-    def do_process_mouse_common(self, proto, wid, pointer, *_args):
+    def do_process_mouse_common(self, proto, device_id, wid:int, pointer, props) -> bool:
+        ss = self._server_sources.get(proto)
+        if not ss:
+            return False
         #adjust pointer position for offset in client:
         try:
             x, y = pointer[:2]
-            assert SetPhysicalCursorPos(x, y)
+            if SetPhysicalCursorPos(x, y):
+                return True
+            #rate limit the warnings:
+            start, count = self.cursor_errors
+            now = monotonic()
+            elapsed = now-start
+            if count==0 or (count>1 and elapsed>10):
+                log.warn("Warning: cannot move cursor")
+                log.warn(" (%i events)", count+1)
+                self.cursor_errors = [now, 1]
+            else:
+                self.cursor_errors[1] = count+1
         except Exception as e:
             log("SetPhysicalCursorPos%s failed", pointer, exc_info=True)
             log.error("Error: failed to move the cursor:")
-            log.error(" %s", e)
-        return pointer
+            log.estr(e)
+        return False
 
-    def clear_keys_pressed(self):
+    def clear_keys_pressed(self) -> None:
         keystate = (BYTE*256)()
         if GetKeyboardState(keystate):
             vknames = {}
@@ -505,20 +502,23 @@ class ShadowServer(GTKShadowServerBase):
                 keystate[x] = 0
             SetKeyboardState(keystate)
 
-    def get_keyboard_config(self, _props=None):
+    def get_keyboard_config(self, _props=None) -> KeyboardConfig:
         return KeyboardConfig()
 
-    def fake_key(self, keycode, press):
+    def fake_key(self, keycode:int, press:bool) -> None:
         fake_key(keycode, press)
 
-    def do_process_button_action(self, proto, wid, button, pressed, pointer, modifiers, *args):
-        self._update_modifiers(proto, wid, modifiers)
-        pointer = self._process_mouse_common(proto, wid, pointer)
+    def do_process_button_action(self, proto, device_id, wid:int, button:int, pressed:bool, pointer, props) -> None:
+        if "modifiers" in props:
+            self._update_modifiers(proto, wid, props.get("modifiers"))
+        #ignore device_id on win32:
+        did = -1
+        pointer = self.process_mouse_common(proto, did, wid, pointer)
         if pointer:
             self.get_server_source(proto).user_event()
-            self.button_action(pointer, button, pressed, -1, *args)
+            self.button_action(did, wid, pointer, button, pressed, props)
 
-    def button_action(self, pointer, button, pressed, deviceid=-1, *args):
+    def button_action(self, device_id, wid:int, pointer, button:int, pressed:bool, props) -> None:
         event = BUTTON_EVENTS.get((button, pressed))
         if event is None:
             log.warn("no matching event found for button=%s, pressed=%s", button, pressed)
@@ -529,13 +529,13 @@ class ShadowServer(GTKShadowServerBase):
         x, y = pointer[:2]
         mouse_event(dwFlags, x, y, dwData, 0)
 
-    def make_hello(self, source):
+    def make_hello(self, source) -> Dict[str,Any]:
         capabilities = GTKServerBase.make_hello(self, source)
         capabilities["shadow"] = True
         capabilities["server_type"] = "Python/gtk2/win32-shadow"
         return capabilities
 
-    def get_info(self, proto, *_args):
+    def get_info(self, proto, *_args) -> Dict[str,Any]:
         info = GTKServerBase.get_info(self, proto)
         info.update(GTKShadowServerBase.get_info(self, proto))
         info.setdefault("features", {})["shadow"] = True
