@@ -1,32 +1,34 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2010-2018 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2023 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os
+import socket
+from typing import Dict, Any
 
 from xpra.util import WORKSPACE_UNSET, get_util_logger
 
 SKIP_METADATA = os.environ.get("XPRA_SKIP_METADATA", "").split(",")
 
 
-def make_window_metadata(window, propname, get_transient_for=None, get_window_id=None, skip_defaults=False):
+def make_window_metadata(window, propname, get_window_id=None, skip_defaults=False) -> Dict[str,Any]:
     try:
-        return do_make_window_metadata(window, propname, get_transient_for, get_window_id, skip_defaults)
+        return do_make_window_metadata(window, propname, get_window_id, skip_defaults)
     except (ValueError, TypeError) as e:
         log = get_util_logger()
         log("make_window_metadata%s",
-            (window, propname, get_transient_for, get_window_id, skip_defaults), exc_info=True)
+            (window, propname, get_window_id, skip_defaults), exc_info=True)
         log.error("Error: failed to make window metadata")
         log.error(" for attribute '%s' of window %s", propname, window)
         log.error(" with value '%s':", getattr(window, propname, None))
-        log.error(" %s", e)
+        log.estr(e)
         return {}
 
 
-def do_make_window_metadata(window, propname, get_transient_for=None, get_window_id=None, skip_defaults=False):
+def do_make_window_metadata(window, propname, get_window_id=None, skip_defaults=False) -> Dict[str,Any]:
     if propname in SKIP_METADATA:
         return {}
     #note: some of the properties handled here aren't exported to the clients,
@@ -39,12 +41,19 @@ def do_make_window_metadata(window, propname, get_transient_for=None, get_window
             if skip_defaults:
                 return {}
             return {propname: ""}
-        return {propname: v.encode("utf-8")}
-    if propname in ("pid", "workspace", "bypass-compositor", "depth", "opacity", "quality", "speed"):
+        return {propname: v}
+    if propname in (
+        "pid", "ppid", "wm-pid",
+        "workspace",
+        "bypass-compositor", "depth", "opacity",
+        "quality", "speed",
+        ):
         v = raw()
         assert v is not None, "%s is None!" % propname
         default_value = {
             "pid"               : 0,
+            "ppid"              : 0,
+            "wm-pid"            : 0,
             "workspace"         : WORKSPACE_UNSET,
             "bypass-compositor" : 0,
             "depth"             : 24,
@@ -56,8 +65,8 @@ def do_make_window_metadata(window, propname, get_transient_for=None, get_window
             return {}
         return {propname : v}
     if propname == "size-hints":
-        #just to confuse things, this is renamed
-        #and we have to filter out ratios as floats (already exported as pairs anyway)
+        # just to confuse things, this attribute is renamed,
+        # and we have to filter out ratios as floats (already exported as pairs anyway)
         v = dict(raw())
         return {"size-constraints": v}
     if propname == "strut":
@@ -68,28 +77,20 @@ def do_make_window_metadata(window, propname, get_transient_for=None, get_window
             strut = strut.todict()
         if not strut and skip_defaults:
             return {}
-        return {"strut": strut}
+        return {propname : strut}
     if propname == "class-instance":
         c_i = raw()
-        if c_i is None:
+        if not c_i:
             return {}
-        return {"class-instance": [x.encode("utf-8") for x in c_i]}
+        return {propname : c_i}
     if propname == "client-machine":
         client_machine = raw()
         if client_machine is None:
-            import socket
             client_machine = socket.gethostname()
-            if client_machine is None:
+            if not client_machine:
                 return {}
-        return {"client-machine": client_machine.encode("utf-8")}
-    if propname == "transient-for":
-        wid = None
-        if get_transient_for:
-            wid = get_transient_for(window)
-        if wid:
-            return {"transient-for" : wid}
-        return {}
-    if propname in ("window-type", "shape", "children"):
+        return {propname : client_machine}
+    if propname in ("window-type", "shape", "children", "hwnd", "relative-position", "requested-position"):
         v = raw()
         if not v and skip_defaults:
             return {}
@@ -126,30 +127,33 @@ def do_make_window_metadata(window, propname, get_transient_for=None, get_window
             return {}
         return {propname : v}
     if propname == "xid":
-        return {"xid" : hex(raw() or 0)}
-    if propname == "group-leader":
-        gl = raw()
-        if not gl or not get_window_id:
+        return {propname : raw() or 0}
+    if propname in ("group-leader", "transient-for", "parent"):
+        ref_window = raw()
+        if not ref_window:
             return  {}
-        xid, gdkwin = gl
         p = {}
-        if xid:
-            p["group-leader-xid"] = xid
-        if gdkwin and get_window_id:
-            glwid = get_window_id(gdkwin)
-            if glwid:
-                p["group-leader-wid"] = glwid
+        if hasattr(ref_window, "get_xid"):
+            #for Gdk X11 windows:
+            p["%s-xid" % propname] = ref_window.get_xid()
+        if get_window_id:
+            wid = get_window_id(ref_window)
+            if wid:
+                if propname=="group-leader":
+                    p["%s-wid" % propname] = wid
+                else:
+                    p[propname] = wid
         return p
     #the properties below are not actually exported to the client (yet?)
     #it was just easier to handle them here
     #(convert to a type that can be encoded for xpra info):
-    if propname in ("state", "protocols"):
-        return {propname : tuple(raw() or [])}
+    if propname in ("state", "protocols", "opaque-region"):
+        return {propname : tuple(raw() or ())}
     if propname == "allowed-actions":
-        return {"allowed-actions" : tuple(raw())}
+        return {propname : tuple(raw())}
     if propname == "frame":
         frame = raw()
         if not frame:
             return {}
-        return {"frame" : tuple(frame)}
-    raise Exception("unhandled property name: %s" % propname)
+        return {propname : tuple(frame)}
+    raise ValueError(f"unhandled property name: {propname}")
