@@ -1,17 +1,23 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2010-2019 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2023 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 #pylint: disable-msg=E1101
 
-import os
 import threading
 from io import BytesIO
-from PIL import Image
+from time import monotonic
+from typing import Dict, Any, Tuple, Union
 
-from xpra.os_util import monotonic_time, load_binary_file, memoryview_to_bytes
+try:
+    from PIL import Image
+except ImportError:
+    Image = None
+
+
+from xpra.os_util import load_binary_file, memoryview_to_bytes
 from xpra.net import compression
 from xpra.util import envbool, envint, csv
 from xpra.log import Logger
@@ -30,9 +36,9 @@ MAX_ARGB_PIXELS = envint("XPRA_MAX_ARGB_PIXELS", 1024)
 """
 Mixin for handling the sending of window icon pixels.
 """
-class WindowIconSource(object):
+class WindowIconSource:
 
-    fallback_window_icon = False
+    fallback_window_icon : Union[bool,Tuple[int,int,str,bytes]] = False
 
     def __init__(self, window_icon_encodings, icons_encoding_options):
         self.window_icon_encodings = window_icon_encodings
@@ -45,7 +51,7 @@ class WindowIconSource(object):
 
         self.window_icon_data = None
         self.send_window_icon_timer = 0
-        self.theme_default_icons = icons_encoding_options.strlistget("default.icons", [])
+        self.theme_default_icons = icons_encoding_options.strtupleget("default.icons")
         self.window_icon_greedy = icons_encoding_options.boolget("greedy", False)
         self.window_icon_size = icons_encoding_options.intpair("size", (64, 64))
         self.window_icon_max_size = icons_encoding_options.intpair("max_size", self.window_icon_size)
@@ -74,7 +80,7 @@ class WindowIconSource(object):
             self.send_window_icon_timer = 0
             self.source_remove(swit)
 
-    def get_info(self):
+    def get_info(self) -> Dict[str,Any]:
         idata = self.window_icon_data
         if not idata:
             return {}
@@ -90,22 +96,67 @@ class WindowIconSource(object):
 
     @staticmethod
     def get_fallback_window_icon():
-        if WindowIconSource.fallback_window_icon is False:
+        if WindowIconSource.fallback_window_icon is False and Image:
             try:
-                from xpra.platform.paths import get_icon_filename
+                from xpra.platform.paths import get_icon_filename  # pylint: disable=import-outside-toplevel
                 icon_filename = get_icon_filename("xpra.png")
-                log("get_fallback_window_icon() icon filename=%s", icon_filename)
-                assert os.path.exists(icon_filename), "xpra icon not found: %s" % icon_filename
+                log(f"get_fallback_window_icon() icon filename={icon_filename}")
                 img = Image.open(icon_filename)
                 icon_data = load_binary_file(icon_filename)
                 icon = (img.size[0], img.size[1], "png", icon_data)
+                img.close()
                 WindowIconSource.fallback_window_icon = icon
                 return icon
             except Exception as e:
-                log.warn("failed to get fallback icon: %s", e)
+                log.warn(f"Warning: failed to get fallback icon: {e}")
                 WindowIconSource.fallback_window_icon = False
         return WindowIconSource.fallback_window_icon
 
+    def get_default_window_icon(self, size=48):
+        #return the icon which would be used from the wmclass
+        wmclass_name = self.get_window_wm_class_name()
+        if not wmclass_name:
+            return None
+        from gi.repository import Gtk
+        it = Gtk.IconTheme.get_default()  # pylint: disable=no-member
+        log("get_default_window_icon(%i) icon theme=%s, wmclass_name=%s", size, it, wmclass_name)
+        for icon_name in (
+            f"{wmclass_name}-color",
+            wmclass_name,
+            f"{wmclass_name}_{size}x{size}",
+            f"application-x-{wmclass_name}",
+            f"{wmclass_name}-symbolic",
+            f"{wmclass_name}.symbolic",
+            ):
+            i = it.lookup_icon(icon_name, size, 0)
+            log("lookup_icon(%s)=%s", icon_name, i)
+            if not i:
+                continue
+            try:
+                pixbuf = i.load_icon()
+                log("load_icon()=%s", pixbuf)
+                if pixbuf:
+                    w, h = pixbuf.props.width, pixbuf.props.height
+                    log("using '%s' pixbuf %ix%i", icon_name, w, h)
+                    return w, h, "RGBA", pixbuf.get_pixels()
+            except Exception:
+                log("%s.load_icon()", i, exc_info=True)
+        return None
+
+    def get_window_wm_class_name(self):
+        try:
+            c_i = self.window.get_property("class-instance")
+        except Exception:
+            return None
+        if not c_i or len(c_i)!=2:
+            return None
+        return c_i[0]
+
+    def client_has_theme_icon(self):
+        wm_class = self.get_window_wm_class_name()
+        return wm_class and wm_class in self.theme_default_icons
+
+
     def send_window_icon(self):
         #some of this code could be moved to the work queue half, meh
         assert self.ui_thread == threading.current_thread()
@@ -115,35 +166,32 @@ class WindowIconSource(object):
         icons = self.window.get_property("icons")
         log("send_window_icon window %s found %i icons", self.window, len(icons or ()))
         if not icons:
-            #this is a bit dirty:
-            #we figure out if the client is likely to have an icon for this wmclass already,
-            #(assuming the window even has a 'class-instance'), and if not we send the default
-            try:
-                c_i = self.window.get_property("class-instance")
-            except Exception:
-                c_i = None
-            if c_i and len(c_i)==2:
-                wm_class = c_i[0].encode("utf-8")
-                if wm_class in self.theme_default_icons:
-                    log("%s in client theme icons already (not sending default icon)", self.theme_default_icons)
-                    return
-                #try to load the icon for this class-instance from the theme:
-                icons = []
-                done = set()
-                for sizes in (self.window_icon_size, self.window_icon_max_size, (48, 64)):
-                    for size in sizes:
-                        if size in done:
-                            continue
-                        done.add(size)
-                        icon = self.window.get_default_window_icon(size)
-                        if icon:
-                            icons.append(icon)
-                log("send_window_icon window %s using default window icon", self.window)
+            if self.client_has_theme_icon():
+                log("%s in client theme icons already (not sending default icon)", self.theme_default_icons)
+                return
+            #try to load the icon for this class-instance from the theme:
+            icons = []
+            sizes = []
+            for size in (self.window_icon_size, self.window_icon_max_size, (48, 64)):
+                if size:
+                    for dim in size:        #ie: 48
+                        if dim not in sizes:
+                            sizes.append(dim)
+            for size in sizes:
+                icon = self.window.get_default_window_icon(size)
+                if icon:
+                    icons.append(icon)
+            if not icons:
+                #try to find one using the wmclass:
+                icon = self.get_default_window_icon()
+                if icon:
+                    log("send_window_icon window %s using default window icon", self.window)
+                    icons.append(icon)
         max_w, max_h = self.window_icon_max_size
         icon = self.choose_icon(icons, max_w, max_h)
         if not icon:
-            #try again, without size restrictions:
-            #(we'll downscale it)
+            # try again, without size restrictions:
+            # (we'll downscale it)
             icon = self.choose_icon(icons)
         if not icon:
             if not self.window_icon_greedy:
@@ -154,7 +202,7 @@ class WindowIconSource(object):
             if self.has_default:
                 #client will set the default itself,
                 #send a mostly empty packet:
-                packet = ("window-icon", self.wid, 0, 0, "default", "")
+                packet = ("window-icon", self.wid, 0, 0, "default", b"")
                 log("queuing window icon update: %s", packet)
                 #this is cheap, so don't use the encode thread,
                 #and make sure we don't send another one via the timer:
@@ -186,11 +234,12 @@ class WindowIconSource(object):
         w, h, pixel_format, pixel_data = idata
         log("compress_and_send_window_icon() %ix%i in %s format, %i bytes for wid=%i",
             w, h, pixel_format, len(pixel_data), self.wid)
-        assert pixel_format in ("BGRA", "RGBA", "png"), "invalid window icon format %s" % pixel_format
+        if pixel_format not in ("BGRA", "RGBA", "png"):
+            raise RuntimeError(f"invalid window icon format {pixel_format}")
         if pixel_format=="BGRA":
             #BGRA data is always unpremultiplied
             #(that's what we get from NetWMIcons)
-            from xpra.codecs.argb.argb import premultiply_argb  #@UnresolvedImport
+            from xpra.codecs.argb.argb import premultiply_argb  #@UnresolvedImport pylint: disable=import-outside-toplevel
             pixel_data = premultiply_argb(pixel_data)
 
         max_w, max_h = self.window_icon_max_size
@@ -205,6 +254,9 @@ class WindowIconSource(object):
 
         image = None
         if must_scale or must_convert or SAVE_WINDOW_ICONS:
+            if Image is None:
+                log("cannot scale or convert window icon without python-pillow")
+                return
             #we're going to need a PIL Image:
             if pixel_format=="png":
                 image = Image.open(BytesIO(pixel_data))
@@ -221,9 +273,13 @@ class WindowIconSource(object):
                     rw = min(max_w, w*icon_h//h)
                     rh = icon_h
                 log("scaling window icon down to %sx%s", rw, rh)
-                image = image.resize((rw, rh), Image.ANTIALIAS)
+                try:
+                    from PIL.Image.Resampling import LANCZOS
+                except ImportError:
+                    from PIL.Image import LANCZOS
+                image = image.resize((rw, rh), LANCZOS)
             if SAVE_WINDOW_ICONS:
-                filename = "server-window-%i-icon-%i.png" % (self.wid, int(monotonic_time()))
+                filename = f"server-window-{self.wid}-icon-{int(monotonic())}.png"
                 image.save(filename, 'PNG')
                 log("server window icon saved to %s", filename)
 
@@ -240,7 +296,8 @@ class WindowIconSource(object):
         self.queue_packet(packet, wait_for_more=True)
 
 
-    def choose_icon(self, icons, max_w=1024, max_h=1024):
+    @staticmethod
+    def choose_icon(icons, max_w=1024, max_h=1024):
         if not icons:
             return None
         log("choose_icon from: %s", csv("%ix%i %s" % icon[:3] for icon in icons))
