@@ -1,26 +1,21 @@
 # This file is part of Xpra.
 # Copyright (C) 2008, 2009 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2012-2020 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2012-2023 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-from xpra.gtk_common.error import trap
-from xpra.x11.bindings.window_bindings import constants     #@UnresolvedImport
-from xpra.x11.gtk_x11.send_wm import send_wm_take_focus     #@UnresolvedImport
+from gi.repository import GObject, Gtk # @UnresolvedImport
+
+from xpra.gtk_common.error import xlog
+from xpra.x11.bindings.window import constants     #@UnresolvedImport
+from xpra.x11.bindings.send_wm import send_wm_take_focus     #@UnresolvedImport
 from xpra.x11.gtk_x11.prop import prop_set
-from xpra.x11.gtk_x11.gdk_bindings import x11_get_server_time
-from xpra.gtk_common.gtk_util import get_default_root_window, screen_get_default, get_xwindow, is_realized
-from xpra.gtk_common.gobject_compat import import_gtk, import_gobject, is_gtk3
+from xpra.gtk_common.gtk_util import get_default_root_window
 from xpra.log import Logger
 
 log = Logger("x11", "window")
 focuslog = Logger("x11", "window", "focus")
 
-
-gtk = import_gtk()
-gobject = import_gobject()
-
-
 XNone = constants["XNone"]
 CurrentTime = constants["CurrentTime"]
 
@@ -87,7 +82,10 @@ CurrentTime = constants["CurrentTime"]
 # have gone wonky.
 
 def root_set(*args):
-    prop_set(get_default_root_window(), *args)
+    root = get_default_root_window()
+    assert root
+    xid = root.get_xid()
+    prop_set(xid, *args)
 
 world_window = None
 def get_world_window():
@@ -102,13 +100,12 @@ def destroy_world_window():
         ww.destroy()
 
 
-class WorldWindow(gtk.Window):
-    def __init__(self, screen=screen_get_default()):
+class WorldWindow(Gtk.Window):
+    def __init__(self):
         global world_window
         assert world_window is None, "a world window already exists! (%s)" % world_window
         world_window = self
-        super(WorldWindow, self).__init__()
-        self.set_screen(screen)
+        super().__init__()
         self.set_title("Xpra-WorldWindow")
         self.set_skip_taskbar_hint(True)
         self.set_skip_pager_hint(True)
@@ -126,21 +123,18 @@ class WorldWindow(gtk.Window):
 
         # Make sure that we are always the same size as the screen
         self.set_resizable(False)
-        screen.connect("size-changed", self._resize)
         self.move(0, 0)
         self._resize()
+        self.get_screen().connect("size-changed", self._resize)
 
-        if is_gtk3():
-            self.add = self._patched_add
-
-    def __repr__(self):
+    def __repr__(self):  #pylint: disable=arguments-differ
         xid = 0
         w = self.get_window()
         if w:
-            xid = get_xwindow(w)
+            xid = w.get_xid()
         return "WorldWindow(%#x)" % xid
 
-    def _resize(self, *_args):
+    def _resize(self, *_args) -> None:
         s = self.get_screen()
         x = s.get_width()
         y = s.get_height()
@@ -164,9 +158,8 @@ class WorldWindow(gtk.Window):
     #      -- it is possible that we should not in fact have the X focus at
     #         this time, though, so then give it to whoever should
     #   -- and finally ignore all subsequent focus-in-events
-    def do_map(self, *args):
-        gtk.Window.do_map(self, *args)
-
+    def do_map(self) -> None:
+        Gtk.Window.do_map(self)
         # We are being mapped, so we can focus ourselves.
         # Check for the property, just in case this is the second time we are
         # being mapped -- otherwise we might miss the special call to
@@ -176,52 +169,52 @@ class WorldWindow(gtk.Window):
             # (ICCCM violating) to use CurrentTime in a WM_TAKE_FOCUS message,
             # but GTK doesn't happen to care, and this guarantees that we
             # *will* get the focus, and thus a real FocusIn event.
-            send_wm_take_focus(self.get_window(), CurrentTime)
+            send_wm_take_focus(self.get_window().get_xid(), CurrentTime)
 
-    def _patched_add(self, widget):
+    def add(self, widget) -> None:
         w = widget.get_window()
         log("add(%s) realized=%s, widget window=%s", widget, self.get_realized(), w)
         #the DesktopManager does not have a window..
         if w:
-            gtk.Window.add(self, widget)
+            super().add(widget)
 
-    def do_focus_in_event(self, event):
+    def do_focus_in_event(self, event) -> None:
         htf = self.get_property("has-toplevel-focus")
         focuslog("world window got focus: %s, has-toplevel-focus=%s", event, htf)
         if not htf:
-            gtk.Window.do_focus_in_event(self, event)
+            Gtk.Window.do_focus_in_event(self, event)
             self.reset_x_focus()
 
-    def do_focus_out_event(self, event):
+    def do_focus_out_event(self, event) -> bool:
         focuslog("world window lost focus: %s", event)
         # Do nothing -- harder:
-        self.stop_emission("focus-out-event")
+        self.stop_emission_by_name("focus-out-event")
         return False
 
-    def _take_focus(self):
+    def _take_focus(self) -> None:
         focuslog("Take Focus -> world window")
-        assert is_realized(self)
+        assert self.get_realized()
         # Weird hack: we are a GDK window, and the only way to properly get
         # input focus to a GDK window is to send it WM_TAKE_FOCUS.  So this is
         # sending a WM_TAKE_FOCUS to our own window, which will go to the X
         # server and then come back to our own process, which will then issue
         # an XSetInputFocus on itself.
         w = self.get_window()
-        now = x11_get_server_time(w)
-        send_wm_take_focus(w, now)
+        send_wm_take_focus(w.get_xid(), CurrentTime)
 
-    def reset_x_focus(self):
+    def reset_x_focus(self) -> None:
         focuslog("reset_x_focus: widget with focus: %s", self.get_focus())
         def do_reset_x_focus():
             self._take_focus()
             root_set("_NET_ACTIVE_WINDOW", "u32", XNone)
-        trap.swallow_synced(do_reset_x_focus)
+        with xlog:
+            do_reset_x_focus()
 
-    def _after_set_focus(self, *_args):
+    def _after_set_focus(self, *_args) -> None:
         focuslog("after_set_focus")
         # GTK focus has changed.  See comment in __init__ for why this isn't a
         # default handler.
         if self.get_focus() is not None:
             self.reset_x_focus()
 
-gobject.type_register(WorldWindow)
+GObject.type_register(WorldWindow)
